<?php
/*-----------------------------------------------------------------------------
** userpref.php - User preferences manager
**
** DESCRIPTION
**
** AUTHOR
**    Fred Hirsch
** COPYRIGHT
**    (C) 2001 Dark Hart Design
**    GNU Public License (see LICENSE file for details)
**-----------------------------------------------------------------------------
*/
/*-----------------------------------------------------------------------------
** Initializing the Interface classes
**    The first step in initialization is to load the core system function and
**    setup file. The "mainfile" loads all the configuration parameters, session
**    handler and database loader for the various sub-elements. Unlike other 
**    paths in the system, the mainfile include must be relative to the location
**    where the current script is running. From the mainfile, the core include
**    path is setup for all the secondary includes.
*/
include_once("../function.php");

/*     The next variables set the interface name and the plugin name of the 
**     current script. The interface name describes shared functionality within
**     a specific plugin, while the plugin name is used to determine the 
**     location of helper scripts to the interfaces.
*/
$interface = 'userpref'; 
$interface_title = 'DarkPortal Admin - User Preferences';
$interface_cols = array('id', 'name', 'value');
$plugin = 'admin';    // articles

/*     We next load plugin specific functions and configuration. Note that these
**     items follow a specific path pattern which must be used to help 
**     standardize the layout of the plugin directories. One exception to this
**     rule is the use of the core admin tools which use their own path pattern.
*/
include("$include_path/$plugin/function.php");
include("$include_path/$plugin/config.php");

if (empty ($userinfo[groups][Administrator])) {
  $reason = "Access to Administration Interface Denied";
  include("$include_path/noaccess.php");
  exit;
}

/*-----------------------------------------------------------------------------
** Interface Initialization
**    The interface class itself is now initialized, the operation variable 
**    used here is always included from the user interface GET or POST request.
**    The Interface constructor creates the global interface variable which 
**    makes it available to the other classes which are initialized by the
**    Initialize method. We cannot initialize in the constructor, as the object
**    is not actually instantiated yet.
*/
isset ($searchby) ? $operation .= ".$searchby" : '';
isset ($form__operation) ? $operation .= ".$form__operation" : '';

/*-----------------------------------------------------------------------------
** >>>>>>>>>>>>>>>>>>>>>>>>>>  END OF GLOBAL CODE <<<<<<<<<<<<<<<<<<<<<<<<<<<<<
*/

/*-----------------------------------------------------------------------------
** CLASS: Interface
**-----------------------------------------------------------------------------
**    The interface class contains all the methods required to handle user 
**    input from the web interface and handles any specialized handling of the
**    display for these requests. This class must always contain a constructor,
**    Initialize(), display(), and main() methods.
*/
class LocalInterface extends Interface{

  /*-----------------------------------------------------------------------------
  ** METHOD: void Interface(string interface, string operation, string plugin)
  **-----------------------------------------------------------------------------
  ** The class constructor requires the input of an interface, operation and
  ** plugin name. Without these elements, the system will only build the default
  ** interface which contains no inherent functionality. These core elements are
  ** used during initialization to create the proper template and theme elements.
  */
  function LocalInterface() {
    $this->themeID = 1;
    $this->flags[transpose] = true;
    $this->Interface();
  }

  /*-----------------------------------------------------------------------------
  ** METHOD: void main(reference template)
  **-----------------------------------------------------------------------------
  ** Almost all interfaces should contain a main() method. This will allow the 
  ** system to override the parsing of main inside the template and perform the
  ** lookup of data which will later be needed to be placed inside the primary
  ** zone of the interface. It is also common to create helper and indexing 
  ** content in other zones. These may be further aided by adding content methods
  ** with the same name as the zone. As an example: It is common to provide 
  ** help features to new users in the rightblock zone of the default theme. If
  ** customized data is needed in these blocks, it could be done in a rightblock()
  ** method.
  */
  function main(&$template) {
    // $dbh is the global database connector object and allows abstract database
    // access and queries. This is included in many cases.
    global $dbh;
    // $config contains the system global configuration array as well as any 
    // special configuration options for the plugin. These are stored as a
    // subarray to config ==> $config[<plugin name>][<element>]
    global $config; 
    // $userinfo contains an array from the user's session information of all
    // information, access control and group ownership assigned to the user. 
    // Base level information is stored by associative keys, while access 
    // control is stored in subarrays. As an example, to test for a user
    // belonging to group "Admin", the following key in the array:
    //     $userinfo[groups][Admin] <<== would be set true or 1.
    global $userinfo;

    // Note the use of references in passing the template around the interface.
    // This is a crucial performance issue, and should always be used in this
    // form. The template can still be modified directly. It is also important
    // to note that any method which overrides the template class to display
    // a specialized version of blocks must have a template object passed to it.
    // otherwise, the receiving method will not have any reference to build 
    // data into the template.
    $this->template =& $template;
    $interface = $this->interface;
    // Assigning data to the template must be done whenever new data has been 
    // loaded, or data is removed, edited or added to the database.
    $template->assign('data', $data);
    // Some blocks are more useful to be added in a repetitive manner. This is
    // especially true when showing summaries of news or other article managers.
    // Performing this operation on the template takes the current block data
    // and applies it to the current main block definition and adds the parsed 
    // block to the template's already parsed information.
    $template->addParsedBlock();
  }
  
  function doEdit() {
    global $dbh;

    $dbfields = array();
    $error = '';
    
    $id = $this->form_data[id];
    $id_q = $this->command->db_quote($this->form_data[id]);
    $plugin_q = $this->command->db_quote($this->form_data[plugin]);
    $hold = $this->form_data[plugin];
    unset($this->form_data[plugin]);
    unset($this->form_data[id]);
    unset($this->form_data[name]);
    while(list($k, $v) = each($this->form_data)) {
      $vq = $this->command->db_quote($v);
      $kq = $this->command->db_quote($k);
      if ($this->command->checkUserPref($k, $id) && $v !== '') {
  		  $sql = "update user_preference set value = $vq, plugin = $plugin_q where name = $kq and id = $id_q";
      } else {
  		  $sql = "insert into user_preference (id, name, value, plugin) values ($id_q, $kq, $vq, $plugin_q)";
      }
 		  $dbh->Execute($sql);
 		  $dbh->ErrorMsg() ? $error = "User preference change failed. ". $dbh->ErrorMsg(): '';
    }
    $this->interface_data[error] = $error;
    $template =& $this->theme->template;
    $this->interface_data = array_merge($this->interface_data, $this->command->getInterfaceData($this->search, true));

    $template->assign('idata', $this->interface_data);
    $this->command->loadUserPrefs();
    $this->form_data[plugin] = $hold;
  }
}

$gui = new LocalInterface();
/*    DarkPortal institutes a common set of CGI parameters which should always
**    be used when performing search or lookup operations for user data. The
**    basic method to create a search pattern is handled by three basic lookup
**    variables:
**    - searchby ==> the search lookup table for the query.
**    - searchid ==> the id to use when performing the search.
**    - searchname ==> If the search needs to lookup on an alternate column, 
**      indicate a name for that column.
**    In future releases, this API will be expanded to allow for ranged 
**    searching, and joins on multiple tables. Currently, the search function
**    is implemented directly in the Interface class. This will be changed to
**    be moved to the mainfile.
*/
//$searchby = 'user_preference';
$gui->Initialize();
/*-----------------------------------------------------------------------------
** Operation Logic
**    Below is a sample of operational logic, which determines what the operation
**    is to be performed, and attempts to perform methods within the GUI to 
**    enact requests by the user. Note that the interface itself will automatically
**    retrieve the appropriate user interface layout blocks from the database 
**    during initialization. User interface methods should never generate output,
**    they should only extract or place data in the database based upon the 
**    user's request.
*/
if (preg_match("/^submit/i", $operation)) {
  // It is typical to set the form data into the interface class if the operation
  // is a submission of information. Form data fields typically take on the form
  // of form_<dataitem>. This transposition may be modified in future releases.
  isset($form_id) ? $gui->form_data[id] = $form_id : '';
  $gui->form_data[name] = $form_name;
  // If an operation does not require input validation, it is placed first.
  if ($form__operation == 'delete') { 
    $gui->doDelete();
  } elseif ($gui->validateForm()){ 
    // Once the form data is placed, it can be validated. 
    // The validation is interface dependent.
    if ($form__operation == 'edit') {
      $gui->doEdit();
    }
  } else {
    if ($form__operation == 'edit') {
      $gui->edit();
    } elseif ($form__operation == 'add') {
      $gui->add();
    } elseif ($form__operation == 'copy') {
      $gui->copy;
    } else {
      $gui->unknownError();
    }
  }
} elseif (eregi("^edit", $operation)) {
  $gui->edit();
}

/*-----------------------------------------------------------------------------
** Interface Display
**    Once the operations (if any) are complete, the display method is called.
**    this enacts the object's theme to display its appropriate blocks.
*/
print $gui->display();
?>
