<?php
/*
* Configuration
*
* While not completely object oriented, this config class opens the door
* for using config as an object in later development. Ideally, the config
* for plugins will eventually inherit from this class, and config will be
* an object owned by interfaces. For now, it creates a simple associative
* array of configuration parameters. This file can be generated dynamically
* from the system config tool and will eventually be generated during the
* standard install process.
*
* @version  0.2.0 beta
* @author   Fred Hirsch
* @copyright  Dark Hart Design  2001
*/

$config = array(
/* dbtype - Database Types
** This portal supports several types of databases. Ensure that if using one of
** the specialized types that the database libraries are installed on the
** system and that PHP has been compiled to use the database.
*/
  	dbtype => "mysql",
/* dbhost - Database Host
** The host name for the database server. In most cases, this is localhost,
** though this may be different depending on your system.
*/
  	dbhost => "localhost",
/* dbuname - Database Username
** Most databases require a username to identify an authenticated user of the
** database.
*/
  	dbuname => "portal",
/* dbpass - Database Password
** While not always required, it is generally recommended that a password be
** used to authenticate the user to the database.
*/
  	dbpass => "",
/* dbname - Database Name
** The actual database name itself. This is usually portal or darkportal. In
** some cases, a user may be forced to use a centralised database name.
*/
  	dbname => "portal",
/* dbtest - Database Test
** This is a test variable that doesn't really do anything at all.
*/
  	system => "UNIX",
/* site_name - Site Name
** A common name for the site will be used to mark many elements of the site.
*/
  	site_name => "Dark Hart Design",
/* site_url - Site URL
** This can be a relative or absolute URL to the site resources. This URL is
** used to build all links for the various portal functions. If the site domain
** will not change often, we suggest using absolute URL values.
*/
  	site_url => "",
/* site_path - Site Path
** This path is the absolute system (Unix/Windows) to the Dark Portal
** installation. A relative path cannot be used as this path is crucial in
** including the various required files for the system.
*/
  	site_path => "/var/www/darkhart.dyndns.org/html",
/* site_theme - Default Theme
** This is the default theme ID used for the site. Individual users may
** override this setting in their personal profiles.
*/
    base_theme => "2",
  	site_theme => "6",
/* site_language - Default Language
** Select the default display language for the site.
*/
  	site_language => "english",
/* site_email - Admin/Site Email
** This email address will be used when notifying users of problems in the site
** as well as being available for use as a display variable.
*/
  	site_email => "admin@foobar.com",
/* default_user_group - Default User Group
** Select the group name that will be used to give default permissions to users
** in the portal.
*/
  	default_user_group => "3",
    track_users => true,
    timezone => 'EST',
    session_expiry => 84600,
    session_crypt => false,
);

function loadPluginConfig() {
  global $dbh, $config, $plugin;
  
  $sql = "select c.name,c.value,c.override from config c, config_group g where c.grp = g.id and g.plugin = '$plugin'";
  $result = $dbh->Execute($sql);

  while ($data = $result->fields) {
    $result->MoveNext();
    if ($data[override] != 1) {
      $config[$plugin][$data[name]] = $data[value];
    } else {
      $config[$data[name]] = $data[value];
    }
  }
}

/**
* Configuration
*
* While not completely object oriented, this config class opens the door
* for using config as an object in later development. Ideally, the config
* for plugins will eventually inherit from this class, and config will be
* an object owned by interfaces. For now, it creates a simple associative
* array of configuration parameters. This file can be generated dynamically
* from the system config tool and will eventually be generated during the
* standard install process.
*
* @version  0.2.0 beta
* @author   Fred Hirsch
* @copyright  Dark Hart Design  2001
* @package CORE
*/
class Config {

/**
* Holds an array containing the global configuration options for the entire 
* portal.
*
* @access private
*/
  var $config = array();

/**
* Config Constructor
* 
* Creates a configuration object from the global configuration. Eventually the
* object methods will replace the static array used above.
*/
  function Config() {
    global $config;
    $this->config = $config;
  }
}
