<?php
// Setup our own internal error handling
$old_error_handler = set_error_handler("CoreErrorHandler");
$error = array();
define ('FATAL', E_USER_ERROR);
define ('WARNING', E_USER_WARNING);
define ('NOTICE', E_USER_NOTICE);
// Magic quotes are bad for our DB routines, so we force them off.
set_magic_quotes_runtime(0);

// Someone might try to hack these core variables, so we explicitly unset them.
unset($include_path);
unset($userinfo);

// A bad attempt at determining our location so we can dynamically choose our
// path. This will eventually be set in the initial setup config.
define('INCLUDE_PATH', "/var/www/darkhart.dyndns.org/html");
$include_path = INCLUDE_PATH;

// Include all the basic modules for the portal system.
include_once("$include_path/config.php");
include_once("$include_path/tables.php");
include_once("$include_path/adodb/adodb.inc.php");
if (extension_loaded('xml')) {
  include_once("$include_path/template.xml.php");
} else {
  include_once("$include_path/template.php");
}
include_once("$include_path/interface.php");
include_once("$include_path/language/lang-$config[site_language].php");
// Send a HTTP header so that the page will never be cached.
header('Expires: Mon, 26 Jul 1997 05:00:00 GMT');
header('Last-Modified: ' . gmdate('D, d M Y H:i:s') . ' GMT');
header('Cache-control: no-cache, must-revalidate');
header('Pragma: no-cache');

// Initialize the database handler and make a persistent connection.
$ADODB_FETCH_MODE = ADODB_FETCH_BOTH; // Implement Assoc only?
ADOLoadCode($config[dbtype]);
$dbh = &NewADOConnection();
if ($config[db_pconnect]) {
	$dbh->PConnect($config[dbhost], $config[dbuname], $config[dbpass], $config[dbname]);
} else {
	$dbh->PConnect($config[dbhost], $config[dbuname], $config[dbpass], $config[dbname]);
}
$SYSTEM_VARS = get_defined_vars();

// Register the session and load the user's info if defined.
$ADODB_SESSION_DRIVER=$config[dbtype];
$ADODB_SESSION_CONNECT=$config[dbhost];
$ADODB_SESSION_USER =$config[dbuname];
$ADODB_SESSION_PWD =$config[dbpass];
$ADODB_SESSION_DB =$config[dbname];
ini_set('session.gc_maxlifetime', $config[session_expiry]);
ini_set('session.cookie_lifetime', 604800);
if ($config[session_crypt] == true) {
  include("$include_path/adodb/adodb-cryptsession.php");
} else {
  include("$include_path/adodb/adodb-session.php");
}
session_name('DarkHart');
session_start();
session_register("userinfo");
if (isset($userinfo[session_expiry]) && $userinfo[session_expiry] > 0) {
  ini_set('session.gc_maxlifetime', $userinfo[session_expiry]);
}

// Load Plugin Method Factory, etc.
if (isset($plugin)) {
  loadPluginConfig();
  if (isset($config[plugin_path]) && $config[plugin_path] != '') {  	
  	include_once("$include_path/{$config[plugin_path]}/function.php");
	  include_once("$include_path/{$config[plugin_path]}/tables.php");
	} else {
		die("Plugin path not defined for $plugin. Plugin is either not properly configured or initialized.");
   }
}
  
/**
** Custom Error Handler
**
** Rather than rely on defauly PHP error handling, we will override the 
** error handler with our own method. Eventually, this error handler will
** write to its own log file (not the DB) and generate proper error 
** messages for display in the interfaces.
**
** @param integer $errno A masked value of the error number.
** @param String $errstr A string describing the error condition.
** @param String $errfile The filename in which the error occured.
** @param integer $errlisne The line number of the program in which the error occurred.
** @param Array $vars An array containing the variables which were set during the error condition.
** @return VOID
**/
function CoreErrorHandler($errno, $errstr, $errfile, $errline, $vars) {
  global $config;
  global $error;
  global $include_path;
  $dt = date("Y-m-d H:i:s (T)");
  $errortypes = array(
                1 => 'Error',
                2 => 'Warning',
                4 => 'Parsing Error',
                8 => 'Notice',
                16 => 'Core Error',
                32 => 'Core Warning',
                64 => 'Compile Error',
                128 => 'Compile Warning',
                256 => 'User Error',
                512 => 'User Warning',
                1024 => 'User Notice');
  if ($errno == 1024 || $errno == 8) { return 0;}
  $error[header] = $errortypes[$errno];
  $error[text] = $error[text] . "<p class=\"error\">$errstr</p>";
  $error[footer] = "<p>Filename: $errfile Line: $errline</p>";
  
  if ($errno == 256 || $errno == 2 || $errno == 16) {
    include("$include_path/error.php");
    exit;
  } 
}

/**
* function.php - Portal Core Method Factory and Interface Class
*
* This module contains the base class for the method command factory 
* and the base class for the interfaces. The API is strongly tied to
* this module in many ways. Only methods which are used universally 
* by all interfaces should be included in this module.
* This module first performs all the module includes and provides a 
* standard header for all dynamic pages. It will then create the universal
* database handler and initialize the user session. Once this is done,
* control is passed back to the local interface for template initialization,
* data and command parsing, and display of the interface.
*
* @version  0.2.0 beta
* @package default
*/

/**
** Method Factory
**
** The base method factory parent contains basic method commands which can be
** executed within an interface. Each local interface initializes the command
** factory by calling on its PluginMethodFactory which is a child of the parent
** below. This allows the Plugins to override the parent if needed and also 
** allows universal commands to be defined for all plugins. Only methods which
** are used in multiple plugins should be defined in the parent. 
**
** @version  0.2.0 beta
** @author   Fred Hirsch
** @copyright  Dark Hart Design  2001
** @package CORE
**/
class MethodFactory {

/**
* Contains the language translation object.
* @access private
*/
  var $language;
/**
* Not implemented
*/
  var $datacache;
/**
* Not implemented
*/
  var $tablecache;
/**
* MethodFactory Constructor
*
* Only called by the plugin method constructor. This constructor creates a 
* new language translation object for the factory. This object will eventually
* be replaced by language elements in the database.
*
* @return VOID
*/
  function MethodFactory() {
		$this->language = new Language();
		$this->datacache = array();
		return $this;
  }
  
/** 
* Database Quote Parser
* 
* Parses text for entry into a database field.
* 
* @param    String    $text  The text to quote for database entry.
* @param    boolean   $flag   The flag determines if Magic Quotes should be considered when quoting the text.
* @return   String    The quoted text with proper special character parsing.
*/
  function &db_quote($text, $flag = true) {
    global $dbh;
    $flag ? $retstring = $dbh->qstr($text, get_magic_quotes_gpc()) : $retstring = $dbh->qstr($text);
    return $retstring;
  }

/**
* Database table abstraction
*
* Allows the datbase tables to be abstracted by containing them in a global
* array. This method of storage should be removed in favor of a class based 
* storage model when possible. This abstraction model allows the use of 
* prefixes on the table names to a single datbase. This assists in allowing the
* tables to be named within the portal while allowing flexability to the setup
* and administration of the database. Only tables that are absolutely needed are
* ever loaded.
*
* @param  String  $table  The name of the table to be extracted.
* @return array   An array containing the table name and an array of columns for the table.
*/
  function &DBGetTable($table) {
    global $dbtable;
    return array($dbtable[$table][table], $dbtable[$table][columns]) ;
  }

/**
* DB Table Column lookup
* 
* Allows only the columns of the named DB table to be extracted for the core 
* table definition.
*
* @param  String  $table The name of the table to be extracted.
* @return array   The array of column values.
*/
  function &DBGetTableColumn($name) {
    global $dbtable;
    return $dbtable[$table][columns];
  }

/**
* DB Table Entire lookup
*
* @return array   Returns the entire database table object as an array.
* @depricated
*/
  function &DBGetAllTable() {
    global $dbtable;
    return $dbtable;
  }

/**
* DB Table Entire lookup
*
* @return array   Returns the entire database table object as an array.
* @depricated
*/
  function &DBGetTables() {
    global $dbtable;
    return $dbtable;
  }

/**
* SQL Error handler
*
* Allows errors from SQL statements to be specially captured and formatted. This
* helps in finding specific errors in SQL statements.
*
* @param  String  $message  The text message related to the SQL error.
* @param  String  $sql      The physical SQL statement that has failed.
* @return String  Outputs the message of the error, formatted depending on the user's privilege to view the error info.
*/
  function sqlError($message, $sql = '') {
    global $dbh, $userinfo;
    if ($userinfo[groups][Administrator]) {
      return "<p class=\"error\">$message</p>\n<p>SQL Error: " . $dbh->ErrorMsg() . "</p>\n<blockquote><i>$sql</i></blockquote>"  ;    
    } else {
      return $message;
    }
  }

/**
* Access Denial Message
*
* Displays access denied message.
*
* @param  String  $msg    The text message displayed as an explaination for access denial.
*/
  function noAccess($msg) {
    global $include_path;

    $reason = $msg;
    include("$include_path/noaccess.php");
    exit;
  }

/**
* User information loader
*
* Loads user information for a given user ID. It loads only core user information,
* and not specific user preferences.
*
* @param  int     $uid  The optional user ID to load data, if not provided, it loads the current user's information.
* @return array   Returns an array of the user information extracted from the database.
*/
  function loadUserInfo($uid = '') {
    global $gui, $userinfo, $dbh, $config;
    
    if ($uid == '') {
        if (isset($userinfo[id]) && $userinfo[id] != '' ) {
            $uid = $userinfo[id];
        } else {
            $gui->interface_data[error] = "Failed to lookup user info, invalid user ID.";
            return false;
        }
    }
    if ($uid != $userinfo[id]) {
        // Check auth priv to see if user is allowed to lookup 
        // other user info.
    }
    $sql = <<<END_SQL
select * from users where id = $uid
END_SQL;
    $result = $dbh->Execute($sql);
    $dbh->ErrorMsg() ? $gui->interface_data[error] .= "Could not load user information." : '';
    return $result->fields;
  }

/**
* Alias info loader
*
* Aliases are the core external presence that user's use to expose their usage
* of the portal. Unlike the user information, alias information is not strictly
* confidential and can be purposely falsified to protect a user's identity.
*
* @param  int     $aid    An optional alias ID for which to load alias info. If no valid ID can be determined, the method exits.
* @return array   Returns an array of alias data from the datbase.
*/
  function loadAliasInfo($aid = '') {
    global $gui, $userinfo, $dbh, $config;
    
    if ($aid == '') {
      $gui->interface_data[error] = "Failed to lookup alias info, invalid alias ID.";
      return false;
    }
    $sql = <<<END_SQL
select * from alias where id = $aid
END_SQL;
    $result = $dbh->Execute($sql);
    $dbh->ErrorMsg() ? $gui->interface_data[error] .= "Could not load alias information." : '';
    return $result->fields;
  }

/**
* Load User Preferences
*
* Based on the global userinfo variable, we will load the user preferences for the
* user from the database. These preferences can be used to customize look and feel
* of individual plugin interfaces or overall display of any interface.
*
* @return VOID
*/
  function loadUserPrefs() {
    global $userinfo;
    global $gui;
    global $dbh;

    $plugin_q = $this->db_quote($gui->plugin);
    if (isset($userinfo[id]) && $userinfo[id] != '') {
      $uid_q = $this->db_quote($userinfo[id]);
    } else {
      return;
    }
    $sql = "select name, value, plugin from user_preference where id = $uid_q and plugin in ('default', $plugin_q)";
    $result = $dbh->Execute($sql);
    $dbh->ErrorMsg() ? $gui->interface_data[error] .= "Authentication failed -- Could not load user preferences." : '';
    while(list($name, $value, $plugin_db) = $result->fields) {
        if ($plugin_db == 'default') {
		    $userinfo[$name] = $value;
		    $userinfo[_SAVED][$name] = true;
		} else {
		    $userinfo[$plugin_db][$name] = $value;
		    $userinfo[_SAVED][$plugin_db][$name] = true;
		}
		$result->MoveNext();
    }
    $userinfo[SAVE_CORE] = false;
    $userinfo[SAVE_PLUGIN] = false;
  }

/**
* Save User Preferences
*
* Saves the currently set user preferences back to the databases. This method was
* specifically designed to allow the interfaces to save preferences for display
* dynamically based on page-to-page changes in user preference without any 
* specific entry in the user preference administration screens. This is done by
* executing the save during finalization of the interface display. Only a specific
* plugin level can be saved at any time. This method will save preferences on two
* levels which are set on prior to calling the method. These levels are set by
* setting the userinfo array values for the keys SAVE_CORE and SAVE_PLUGIN. If
* SAVE_CORE is set, all core preferences are automatically saved. If SAVE_PLUGIN
* is set, all plugin specific preferences are saved.
*
* @param  int     $uid    The user ID for which to save preferences. Generally always should be provided.
* @param  String  $plugin The plugin for which the preferences are saved. If not provided, will save prefs based on current plugin.
*/
    function saveUserPrefs($uid = '', $plugin = '') {
        global $userinfo, $gui, $dbh;
        $id = (isset($id) && $id != '') ? $id : $userinfo[id];
        $plugin = (isset($plugin) && $plugin != '') ? $plugin : $gui->plugin;
        $id_q = $this->db_quote($id);

        $dbtable = $this->DBGetTables(); 
        $cols = $dbtable[user_preference_column];

        if ($id < 1) { return false; }
        if ($userinfo[SAVE_CORE]) {
            $plugin_q = $this->db_quote('default');
            while (list($k, $v) = each ($userinfo)) {
                $k_q = $this->db_quote($k);
                $v_q = $this->db_quote($v);
                if ($k == $gui->plugin) { continue; }
                if ($userinfo[_SAVED][$gui->plugin][$k]) {
                    $sql = <<<END_SQL
update $dbtable[user_preference]
    set $cols[name] = $k_q,
    $cols[value] = $v_q
    where id = $id_q
    and plugin = $plugin_q
END_SQL;
                } else {
                    $sql = <<<END_SQL
insert into $dbtable[user_preference]
($cols[id], $cols[name], $cols[value], $cols[plugin])
values
($id_q, $k_q, $v_q, $plugin_q)
END_SQL;
                }
                $dbh->Execute($sql);
                $dbh->ErrorMsg() ? $gui->interface_data[error] .= "Error saving user preferences." : '';
            }
        }
        if ($userinfo[SAVE_PLUGIN]) {
            $plugin_q = $this->db_quote($plugin);
            while (list($k, $v) = each ($userinfo[$plugin])) {
                $k_q = $this->db_quote($k);
                $v_q = $this->db_quote($v);
                if ($userinfo[_SAVED][$plugin][$k]) {
                    $sql = <<<END_SQL
update $dbtable[user_preference]
    set $cols[value] = $v_q
    where id = $id_q
    and plugin = $plugin_q
    and name = $k_q
END_SQL;
                } else {
                    $sql = <<<END_SQL
insert into $dbtable[user_preference]
($cols[id], $cols[name], $cols[value], $cols[plugin])
values
($id_q, $k_q, $v_q, $plugin_q)
END_SQL;
                }
                $dbh->Execute($sql);
                $dbh->ErrorMsg() ? $gui->interface_data[error] .= "Error saving user preferences." : '';
            }
        }
        unset ($userinfo[_SAVED]);
    }

/**
* Check for user level bans
*
* Each interface may check for specific bans a user may have instituted against
* their location or account. Bans occur on two levels, the user ID may be banned
* or their location or IP address may be specifically banned. Bans can occur on
* the portal or plugin level, so that specific users can be banned from the 
* entire site, or just certain areas. Bans also may expire over time, and this
* is monitored and adjusted by the check automatically.
* TODO This method does not yet allow for pattern matching on the banned IP.
* 
* @param  String  $plugin The name of the plugin to check for bans.
* @return boolean Returns true if the user is banned, false if not.
*/
  function checkUserBan($plugin = '') {
    global $dbh, $userinfo, $config;

    list($dbt_ban, $bcol) = array_values($this->DBGetTable('user_ban'));
	
    $ban_dt = $this->db_quote($this->formatUNIXTimestamp());
	  $sql = "delete from $dbt_ban where $bcol[expiry_date] <= $ban_dt";
		$result = $dbh->Execute($sql) or trigger_error('Could not properly expire old bans',FATAL);

		$uid_q = $this->db_quote($userinfo[id]);
		$ip_q = $this->db_quote($userinfo[ip_address]);
    $plug_q = $this->db_quote($plugin);

    //CHECK: See if the user's IP address has been banned
    $sql = <<<END_SQL
select $bcol[reason], $bcol[expiry_date]
from $dbt_ban
where $bcol[expiry_date] > $ban_dt
and $bcol[plugin] = $plug_q
and ($bcol[uid] = $uid_q or $bcol[ip] = $ip_q)
order by $bcol[expiry_date]
END_SQL;
    $result = $dbh->Execute($sql) or trigger_error('Could not read the user ban table', FATAL);
    $data = $result->fields;
    if (isset($data[expiry_date]) && $data[expiry_date] != '') {
        trigger_error ( "This account has been banned from using the $plugin plugin. This account ban will expire on " . $this->command->formatDBTimestamp($data[expiry_date]) . ". The reason for this ban: $data[reason]", FATAL);
        return true;
	  }
    return false;    
  }
  
/**
* Load Configuration
*
*/
  function loadConfig() {
    global $dbh, $config, $gui;
    $plugin_q = $this->db_quote($gui->plugin);
    $sql = "select c.name, c.value from config c, config_group g where g.plugin = $plugin_q and g.id = c.grp";
    $result = $dbh->Execute($sql);
    $config[sysdate] = $this->formatDBTimestamp('', true);
    while ($data = $result->fields) {
      $result->MoveNext();
      $sub = explode(".", $data[name]);
      eval("\$data[value] = \"$data[value]\"; ");
      if (sizeof($sub) == 1) {
        $config[$gui->plugin][$data[name]] = $data[value];
      } else {
        $arr_val[$sub[sizeof($sub) - 1]] = $data[value];
        for ($i = sizeof($sub) - 2 ; $i >= 0; $i--) {
          $arr_val[$sub[$i]] = $arr_val;
        }
        array_push($config[$gui->plugin],$arr_val);
      }

    }
  }
/**
* Load Search
*
* Using the global search parameters, the method will create a private array of
* search parameters for generating an interface data query.
*
* @param 		global Array		$interface_cols 		The column names used for extracting data. Set by the interface.
* @param		global String		$searchby		Table name to search for data.
* @param 		global String 	$searchcols		Table columns for data, set by interface/CGI query.
* @param 		global String		$searchname		Name of the ID column to search by.
*	@param		global String		$searchid			Value of the ID to search for data.
*	@param		global String		$searchrange	Can be used to search for a range in the ID column rather than an exact value.
* @param		global String		$searchorder	Redefines the order column that the search will use.
* @param		global String 	$searchpattern		A search expression pattern which uses %.% to search for data, where % is a wildcard marker and . is the replacement marker for the search ID.
* @param 		global String		$searchone		Will restrict the search using a distinct mapping.
* @param		global String		$searchextra	Uses standard SQL format to add additional search parameters.
* @return		Array		Returns an an array of the search parameters in a format used by the data loader.
*/
  function &loadSearch() {
    global $gui;
    global $interface_cols; // 
    global $searchby;       // Search 
    global $searchcols;     // 
    global $searchname;     // 
    global $searchid;       // 
    global $searchrange;    // 
    global $searchorder;    // 
    global $searchpattern;
    global $searchexact;
    global $searchone;
    global $searchextra;

    empty($gui->search) ? $search = array() : $search = $gui->search;
    $search[key] = (isset($search[key]) && $search[key] != '') ? $search[key] : $searchby;
    if (isset ($search[key]) && $search[key] != '') {
      $interface_cols = $this->getColumnNames($search[key]);
    }

    empty($search[id]) ? $search[id] = $searchid : '';
    if (empty($search[name])) {
      $search[name] = empty($searchname) ? "id" : $searchname;
    }
    if (empty($search[cols])) {
      $search[cols] = empty($searchcols) ? $interface_cols : explode('|', $searchcols);
    }
    empty($search[range]) ? $search[range] = explode('|', $searchrange) : '';
    empty($search[order]) ? $search[order] = $searchorder : '';
    empty($search[pattern]) ? $search[pattern] = $searchpattern : '';
    empty($search[exact]) ? $search[exact] = $searchexact : '';

    if (isset($search[exact]) && $search[exact] == 1) { $search[pattern] = ''; }
    
    // Distinct or search for one will perform a lookup of singular
    // data items when they may repeat.
    if (empty($search[distinct])) {
      if (isset($searchone) && $searchone == 'true') {
        $search[distinct] = 'DISTINCT';
      }
    }
    
    // Extra search items allow additional constraints to be placed
    // on the SQL data search. This is important in differentiating 
    // between data items with identical names, but keyed to alternate
    // ID methods.
    if (empty($search[extra]) && isset($searchextra)) {
      while(list(, $v) = each($searchextra)) {
        list($key, $val) = explode('|', $v);
        $search[extra][$key] = $val;
      }
    }
    $search[loaded] = true;
    return $search;
  }
  
  function &getColumnNames($dbtable) {
    global $dbh;
    $return_colnames = array();
		$columns = $dbh->MetaColumns($dbtable);
		while (list(,$column_obj) = each($columns)) {
			$return_colnames[] = $column_obj->name;
		}
		return $return_colnames;
  }
/**
* Load Form Data
*
* Dark Portal uses a standard method for determining names of variables used in 
* an HTML form. All form variables can be used in the form templates to gather
* previously used data, or for validation of form information. These variables
* are also used to persist data to the database as needed. Form data variables 
* should have identical names to those columns for which they are destined. 
* Special form variables exist using double underscores "__" to indicate that they
* are informational only, and are not intended for database persistence.
*
* @param		global Array 		$SYSTEM_VARS		The method reads all system variables since some interfaces may wish to override the form variable from the CGI entry.
* @return		Array		An array of the form data keyed by the field name.
*/
  function &loadFormData() {
    global $SYSTEM_VARS;
    
    $form_data = array();
    while (list ($k, $v) = each ($SYSTEM_VARS)) {
      if (preg_match("/^form_([^_].+)/", $k, $match) ) {
				$form_data[$match[1]] = $v;
      }
    }
    $SYSTEM_VARS = array();
    return $form_data;
  }

/**
* Get Interface Data
*
* Loads the data from the searched database table into an array which is used
* in many global interface operations. By default, this array is also loaded
* directly into the interface template and used for variable replacement in the
* various template blocks and elements.
*
* @param		Array 		$search 		An array of the search variables, generated by Load Search Data
* @param 		boolean		$transpose	A flag which will force the data to be transposed from a vertical to horizontal format. Used for loading preferences and other similar data into singular arrays.
* @return		Array		Returns either a single row or a multidimensional array of many rows of data.
* @see 	loadSearch
*/
  function &getInterfaceData($search, $transpose = false) {
    global $dbh;
    global $gui;
    $idata = array();
    $noparse = array();
    
    $sql = "select $search[distinct] " . join (", ", $search[cols]) . " from $search[key]";
    if (isset($search[id])) {
      if (isset($search[pattern]) && $search[pattern] != '') {
      	$search[id] = preg_replace ("/(\%?)\.(\%?)/", '\\1' . $search[id] . '\\2', $search[pattern]);
      	$sql .= " where ($search[name] like " . strtoupper($this->db_quote($search[id])) . ")";
      	$sql .= " or ($search[name] like " . strtolower($this->db_quote($search[id])) . ")";
      } else {
      	$sql .= " where $search[name] = " . $this->db_quote($search[id]);
      }
    } elseif (isset($search[range]) ) {
      $sql .= " where $search[name] between {$search[range][0]} and {$search[range][1]}";
    }
    if (isset($search[extra]) ) {
      while (list($k, $v) = each($search[extra])) {
        $v = ($v != '_ALL_') ? $v : '';
        $v_q = $this->db_quote($v);
        $sql .= " and $k = $v_q\n";
      }
      
    }
    if (isset($search[order]) ) {
      $sql .= " order by $search[order]";
    }
    $result = $dbh->Execute ($sql);
    !$result ? die("SQL: $sql -- " . $dbh->ErrorMsg()) : '';

    $count = 0;
    while ($data = $result->fields) {
      $result->MoveNext();
      $idata[$count] = $data;
      
      while (list($k, $v) = each($data)) {
        $noparse[$count][$k] = htmlentities($v);
      } 
      ++$count;
    }
    if (sizeof($idata) == 1) {
      $idata = $idata[0];
      $idata[_NOPARSE] = $noparse[0];
    }
    if ($transpose == true) {
    	$idata = $this->transposeData($idata);
    	$noparse = $this->transposeData($noparse);
    }
    return $idata;
  }

/**
* Parse Content
* 
* Some named fields may require special parsing for display in the interfaces.
* Specifically, the admin interfaces require that the content of the blocks and
* elements be specially parsed to display the tags without parsing those tags
* into their respective data elements. This is done by mystically converting the
* internally used tags into an external representation.
* TODO: This method only handles a single row of data.
* 
* @param 	Object	$theme		A reference to the current theme object, used to get a list of current theme tags.
*	@param	Array		$data			An array of data to be parsed against.
*/
  function &parseContent(&$theme, &$data) {
    if (is_array($data[0])) {
      for($i = 0; $i < sizeof($data); $i++) {
        if (isset($data[$i][content])) {
          $data[$i][html][content] = preg_replace("/\{({$theme->tconfig[tags]})([^}]*)}/", "<[\\1\\2]>", $data[$i][content]);
          $data[$i][html][content] = htmlentities($data[$i][html][content]);
          $data[$i][content_html] = $data[$i][html][content];

          $data[$i][parsed][content] = $this->contentParser($data[$i][html][content], $data[$i]);
          $data[$i][content_parsed] = $data[$i][parsed][content];
        }
        if (isset($data[$i][blockdef])) {
          $data[$i][blockdef_html] = $data[$i][blockdef];
          $data[$i][blockdef_html] = preg_replace("/\{({$theme->tconfig[tags]})([^}]*)}/", "<[\\1\\2]>", $data[$i][blockdef_html]);
          $data[$i][blockdef_html] = htmlentities($data[$i][blockdef_html]);
          $data[$i][blockdef_parsed] = $this->contentParser($data[$i][blockdef_html], $data[$i]);
        }
      }
    } else {
      if (isset($data[content])) {
        $data[html][content] = preg_replace("/\{({$theme->tconfig[tags]})([^}]*)}/", "<[\\1\\2]>", $data[content]);
        $data[html][content] = htmlentities($data[html][content]);
        //$data[html][content] = $this->contentParser($data[html][content], $data);
        $data[content_html] = $data[html][content];
        
        $data[parsed][content] = $this->contentParser($data[html][content], $data);
        $data[content_parsed] = $data[parsed][content];
      }
      if (isset($data[blockdef])) {
        $data[blockdef_html] = $data[blockdef];
        $data[blockdef_html] = preg_replace("/\{({$theme->tconfig[tags]})([^}]*)}/", "<[\\1\\2]>", $data[blockdef_html]);
        $data[blockdef_html] = htmlentities($data[blockdef_html]);
        $data[blockdef_parsed] = $this->contentParser($data[blockdef_html], $data);
      }
    } 
  }

  function parseTags ($text, $markup = 0, $html = 0, $image = 0) {
    global $config;
    global $userinfo;
      include_once (INCLUDE_PATH . "/taglib.php");
      $parser = new YBBTags($text, $this->getSmiles(), "$config[site_url]/images/", $userinfo[id]);
      $parser_ops = ($html == 1) ? $parser->OP_SAFEHTML : $parser->OP_NOHTML;
      $parser_ops .= $parser->OP_PHP . $parser->OP_NBSP . $parser->OP_NL2BR;
      $parser_ops .= ($markup == 1) ? $parser->OP_SIMPLE . $parser->OP_SMILIES . $parser->OP_LIST . $parser->OP_APPEARANCE . $parser->OP_ALIGN . $parser->OP_CODE . $parser->OP_QUOTE . $parser->OP_URLS . $parser->OP_MISC : '';
      $parser_ops .= ($image == 1) ? $parser->OP_IMAGES : '';
      $text = $parser->pgetText($parser_ops);
    return $text;
  }

/**
* Transpose Data
*
* Data in the form of 
*     ID    NAME    VALUE 
*     1     name1   val1
*     2     name2   val2
*     3     name3   val3
* Will be transposed into
*     name1   name2   name3
*     val1    val2    val3
*
* @param  Array   $data   An array of the data to be transposed.
* @return   Array   An array of the transposed data.
*/
  function &transposeData(&$data) {
    $data_new = array();
    $data_new[id] = $data[0][id];
    for ($i = 0; $i < sizeof($data); $i++) {
      $data_new[$data[$i][name]] = $data[$i][value];
    }
    return $data_new;
  }
  function buildURL($data_keys, $file, $path, $opers) {
    $urls = array();

    while (list($k, $v) = each($data_keys)) {
      $opts .= "&form_$k=$v";
    }

    while (list($k, $v) = each($opers)) {
      $urls[$v] = "$path/$file?operation=$v$opts";
    }
    return $urls;
  }
/**
* Format Database Timestamp
*
* Dark Portal forces all timestamps to be set in the database as VARCHAR
* fields rather than database date or time formats. This allows ultimate
* control for the portal in how dates can be interpretted and stored. This
* also prevents problems in handling system level timestamps. The database 
* timestamp is always formatted as YYYYMMDDHHMISS.
*
* @param    String    $time   The database formatted timestamp (YYYYMMDDHHMISS).
* @return   String    Returns a LOCALE specific timestamp as determined by the language object.
*/
  function formatDBDate($time) {
  	if ($time == '30010101000000') {
      return 'Never';
    }
    
    preg_match("/([0-9]{4})([0-9]{1,2})([0-9]{1,2})([0-9]{1,2})([0-9]{1,2})([0-9]{1,2})/", $time, $datetime);
    $datemake = mktime($datetime[4],$datetime[5],$datetime[6],$datetime[2],$datetime[3],$datetime[1]);
    $datetran = "%B %d, %Y";
    $datetime = strftime($datetran, $datemake);
    $datetime = ucfirst($datetime);
    return($datetime);
	}

  function formatDBTime($time) {
  	if ($time == '30010101000000') {
      return 'Never';
    }
    
    preg_match("/([0-9]{4})([0-9]{1,2})([0-9]{1,2})([0-9]{1,2})([0-9]{1,2})([0-9]{1,2})/", $time, $datetime);
    $datemake = mktime($datetime[4],$datetime[5],$datetime[6],$datetime[2],$datetime[3],$datetime[1]);
    $datetran = "%I:%M:%S %p";
    $datetime = strftime($datetran, $datemake);
    $datetime = ucfirst($datetime);
    return($datetime);
	}

  function formatDBTimestamp($time = '', $flag = false) {
    if ($time == '') { $time = date("YmdHis", time()); }
    if ($time == '30010101000000') {
      return 'Never';
    }
    
    preg_match("/([0-9]{4})([0-9]{1,2})([0-9]{1,2})([0-9]{1,2})([0-9]{1,2})([0-9]{1,2})/", $time, $datetime);
    $datemake = mktime($datetime[4],$datetime[5],$datetime[6],$datetime[2],$datetime[3],$datetime[1]);
    $datetran = $this->language->translate("datestring");
    $datetime = strftime($datetran, $datemake);
    $datetime = ucfirst($datetime);
    return($datetime);
  }

/**
* Format UNIX/System Timestamp
*
* Extracts a UNIX level timestamp (seconds since 1970) and converts the value
* to an internal timestamp format (YYYYMMDDHHMISS).
* 
* @param    String    $time   A UNIX/System timestamp value or blank. Blank will extract the current timestamp from the system time() call.
* @return   String    A portal specific timestamp format.
*/
  function formatUNIXTimestamp($time = '') {
    ($time == '') ? $time = time() : '';
    return date("YmdHis", $time);
  }

/**
* Date Array
* 
* Using an internal portal date format, the system will return an array
* variable containing the values for the date. The form generation code
* uses this method to create a value object for date dropdown box formats.
*
* @param    String    $date   An internal portal date string/timestamp. If blank, the method returns a dummy set of neutral blank data.
* @return   Array   An array containing the date information, separated by named key.
*/
	function &dateArray($date = '') {
		if ($date == '') {
			$date_elements = array (year => '',
															month => '',
															day => '',
															hour => '',
															minute => '',
															second => '');
		} else {
			$date_elements = array (year => substr($date, 0, 4),
															month => substr($date, 4, 2),
															day => substr($date, 6, 2),
															hour => substr($date, 8, 2),
															minute => substr($date, 10, 2),
															second => substr($date, 12, 2));
		}
		return $date_elements;
	}
	
/**
* Manipulate Time Values
*/
	function manipulateTime ($date = '', $amount, $interval) {
	  if ($date == '') { $date = $this->formatUNIXTimestamp(); }
	  $da = $this->dateArray($date);
	  
    $da[month]++;
	  if ($interval == 'Y' or $interval == 'y') {
	    $da[year] += $amount;
	  } elseif ($interval == 'M' or $interval == 'm') {
	    $da[month] += $amount;
	  } elseif ($interval == 'D' or $interval == 'd') {
	    $da[day] += $amount;
	  } elseif ($interval == 'H' or $interval == 'h') {
	    $da[hour] += $amount;
	  } elseif ($interval == 'I' or $interval == 'i') {
	    $da[minute] += $amount;
	  } elseif ($interval == 'S' or $interval == 's') {
	    $da[second] += $amount;
    }	  
    $ts = mktime($da[hour], $da[minute], $da[second], $da[month] - 1, $da[day], $da[year]); 
    return date("YmdHis", $ts);
  }

/**
*
*/
  function calculateSeconds($startTime, $endTime) {
    return $this->createTimestamp($endTime) - $this->createTimestamp($startTime);
  }
  
  function createTimestamp($timestring) {
  	$da = $this->dateArray($timestring);
  	return mktime($da[hour], $da[minute], $da[second], $da[month], $da[day], $da[year]);
  }
  
/**
* Month Array
*
* Returns an array which contains a list of month names. Used primarily by
* date display functions which rely on month names instead of value.
*
* @return   Array   An array of month names ordered at zero.
*/
	function monthArray() {
		$months = array('', 'January', 'February', 'March', 'April', 'May', 'June',
										'July', 'August', 'September', 'October', 'November',
										'December');
		return $months;
	}
		
/**
* Track User Locations
*
* If desired, the chat is capable of tracking user location on the portal
* and reporting this information in a number of ways. The most evident is
* the user's who's online plugin which shows basic user locale info.
*/
  function trackUser(&$gui) {
    global $userinfo;
    global $dbh;
    $sql = '';
    $sid_q = $this->db_quote(session_id());
    $date = $this->formatUNIXTimestamp();
    $date_q = $this->db_quote( $date );
    $date_exp_q = $this->db_quote($this->manipulateTime($date, 1, 'I'));
    $login_q = ($userinfo[id] > 0) ? 1 : 0;
    $uid_q = $this->db_quote($userinfo[id]);

      $sql = <<<END_SQL
    insert into user_tracking
    (sid, plugin, extra, expire_date, login_flag, create_date, uid, update_date)
    values
    ($sid_q,'{$gui->plugin}','{$gui->tracking}',$date_exp_q, $login_q, $date_q, $uid_q, $date_q)
END_SQL;
    $dbh->Execute($sql);

    // If the insert wasn't successful, try an update. (This may not be exactly efficient).
    if ($dbh->ErrorMsg()) {
      $sql = <<<END_SQL
    update user_tracking
    set plugin = '{$gui->plugin}',
    extra = '{$gui->tracking}',
    expire_date = $date_exp_q,
    login_flag = $login_q,
    uid = $uid_q,
    update_date = $date_q
    where sid = $sid_q
END_SQL;
      $dbh->Execute($sql);
      $dbh->ErrorMsg() ? die ("DB ERROR: " . $dbh->ErrorMsg() ) : '';
    }
    // Then delete the old records.
    $sql = <<<END_SQL
    delete from user_tracking 
    where expire_date <= $date_q
END_SQL;
    $dbh->Execute($sql);
    $dbh->ErrorMsg() ? die ("DB ERROR: " . $dbh->ErrorMsg() ) : '';
  }
  
/**
* Get User Name
* 
* Based on a userid number, this method will return the displayable name of
* the user to the calling method. This name may be either the real or alias 
* name.
*/
	function getUserName($id = '') {
		global $dbh;
		global $userinfo;
		
		if ($id == '') {
		  if (isset($userinfo[alias]) && $userinfo[alias] != '') {
		    return $userinfo[alias];
		  } else {
		    return $userinfo[name];
		  }
		} else {
			$id = $this->db_quote($id);
		  $sql = "select name, alias from users where id = $id";
		  $result = $dbh->Execute($sql);
		  if (isset($result->fields[alias]) && $result->fields[alias] != '') {
		    return $result->fields[alias];
		  } else {
		    return $result->fields[name];
		  }
		}
	}
	
/**
* Get User Email
*
* Based on the user id number, this method will return the displayable email
* of the user. This email may be either the real or fake email.
*/
	function getUserEmail($id) {
		global $dbh;
		$id = $this->db_quote($id);
	  $sql = "select email, fakeemail from users where id = $id";
	  $result = $dbh->Execute($sql);
	  if (isset($result->fields[fakeemail]) && $result->fields[fakeemail] != '') {
	    return $result->fields[fakeemail];
	  } else {
	    return $result->fields[email];
	  }
	}
		
/**
* RSS UltraMode - Not Implemented
*/
  function ultramode() {
  }
  
/**
* Censor Content Filter
* 
* @deprecated
*/
  // Checks words in a message against words in a censor list and censors it.
  function censorContent($content) {
  }
  
/**
* Send Email Message
*/
	function sendEmail ($addr, $message) {
		//print "###$addr<br>";
	}
	
/**
* Return an array of smile tags used for the YPP tag parser
*/
  function &getSmiles () {
    global $config, $dbh;
    
    list($dbtable,$icol) = array_values($this->DBGetTable('icon_smiley'));
    
    $sql = "select $icol[id], $icol[filename], $icol[code] from $dbtable order by $icol[ord]";
    $result = $dbh->Execute($sql) or die ("Could not load smiley icon definition $sql");
    $smiles = array();
    while ($data = $result->fields) {
      $result->MoveNext();
      $icons = unserialize($data[code]);
      while (list($k, $v) = each ($icons)) {
        $smiles[$v] = "$config[site_url]/images/emoticon/$data[filename]";
      }
    }
    return $smiles;    
  }

  function getNameFromID($id, $table, $col = 'name') {
    global $dbh;
    if (isset($this->datacache[$table][$col][$id]) ) {
      return $this->datacache[$table][$col][$id];
    }
    $id_q = $this->db_quote($id);
    $sql = "select $col from $table where id = $id_q";
    $result = $dbh->Execute($sql);
    $this->datacache[$table][$col][$id] = $result->fields[$col];
    $retval = $result->fields[$col];
    return $retval;
  }
}

?>
