<?php
/*
* Chat Alias Interface
*
* This interface allows the adding, deleting and modification of chat
* aliases. 
*
* @version  0.2.0 beta
* @author   Fred Hirsch
* @copyright  Dark Hart Design  2001
* @package API
*/

/**
* Initializing the Interface classes
*    The first step in initialization is to load the core system function and
*    setup file. The "mainfile" loads all the configuration parameters, session
*    handler and database loader for the various sub-elements. Unlike other 
*    paths in the system, the mainfile include must be relative to the location
*    where the current script is running. From the mainfile, the core include
*    path is setup for all the secondary includes.
*/
include_once("../../function.php");
/*     The next variables set the interface name and the plugin name of the 
*     current script. The interface name describes shared functionality within
*     a specific plugin, while the plugin name is used to determine the 
*     location of helper scripts to the interfaces.
*/
$interface = 'chat.alias'; 
$interface_title = 'DarkPortal - Chat Alias Manager';
$interface_cols = array('id', 'name', 'description', 'system', 'status', 'activity',
                        'url', 'image', 'entry_message', 'exit_message', 'allow_public',
                        'can_modify');
$plugin = 'chat';    // Plugin Name

/*     We next load plugin specific functions and configuration. Note that these
*     items follow a specific path pattern which must be used to help 
*     standardize the layout of the plugin directories. One exception to this
*     rule is the use of the core admin tools which use their own path pattern.
*/
include("$include_path/plugins/$plugin/function.php");
include("$include_path/plugins/$plugin/config.php");

/*-----------------------------------------------------------------------------
* Authentication Handling
*    In the case that a specific interface should limit access to its functions,
*    the following code sample can be used to limit access and provide the user
*    with a reason for the denial. This sample could be expanded to force the
*    user to be a member of specific group(s) or use a plugin config variable
*    to turn on group level authentication.
*/
if ($config[chat][guest_entry] == false && empty($userinfo[groups][User])) {
  $reason = "You must be logged to a valid chat access account in to use the chat system.";
  include("$include_path/noaccess.php");
  exit;
}

isset ($searchby) ? $operation .= ".$searchby" : '';
isset ($form__operation) ? $operation .= ".$form__operation" : '';

/*-----------------------------------------------------------------------------
* >>>>>>>>>>>>>>>>>>>>>>>>>>  END OF GLOBAL CODE <<<<<<<<<<<<<<<<<<<<<<<<<<<<<
*/

/**
* API Interface 
*
* The following template provides the basic level of functionality for all
* Dark Portal plugin interfaces. It serves as a common interface or API to
* perform all display functions for Dark Portal addons and current core
* functionality. For detailed developer instructions, refer to comments in
* this file, and the developer manual included with Dark Portal.
* @version  0.2.0 beta
* @author   Fred Hirsch
* @copyright  Dark Hart Design  2001
* @package API
*/
class LocalInterface extends Interface {

/**
* API Interface Constructor
*
* The local interface contructor should usually only call the super class
* constructor. In some cases, it may be useful to add some parameters such
* as static theme ID (admin interfaces) and so forth.
*
* @return VOID
*/
  function LocalInterface() {
    $this->Interface();
  }

  function initializeLocal() {
    global $dbh;
    global $userinfo;

    if (isset($this->interface_data[id])) {
      $sql = "select name, value from chat_alias_extra where aid = {$this->interface_data[id]}";
      $result = $dbh->Execute($sql);
      $dbh->ErrorMsg() ? $this->interface_data[error] =  "Cannot find extra information for {$this->interface_data[name]}." : '';
      
      while ($adata = $result->fields) {
        $result->MoveNext();
        $this->interface_data['ex_' . $adata[name]] = $adata[value];
      }
      $this->theme->template->assign('idata', $this->interface_data);
    
      $this->interface_data[system_d] = $this->command->getNameFromID($this->interface_data[system], 'chat_alias_system');
      $this->interface_data[status_d] = $this->command->getNameFromID($this->interface_data[status], 'chat_alias_status');
      $this->interface_data[activity_d] = $this->command->getNameFromID($this->interface_data[activity], 'chat_alias_activity');
    }
  }
  
/**
* Main Zone Callback Method
*
* @param ObjectRef $template A reference to the template object to use for assigning template date, etc.
* @return VOID
*/
  function main(&$template) {
    $template->assign('idata', $this->interface_data);
  }

  function doAdd() {
    global $dbh;
    global $userinfo;
    
    $this->form_data[uid] = $userinfo[id];
    $this->form_data[can_modify] = 1;
    $this->form_data[create_date] = $this->command->formatUNIXTimestamp();
    $this->form_data[modify_by] = $userinfo[id];
    $this->form_data[modify_date] = $this->command->formatUNIXTimestamp();
    $this->form_data[login_date] = '00000000000000';
    $this->form_data[delete_date] = '00000000000000';
    $this->form_data[inactive_date] = '00000000000000';
    unset ($this->form_data[id]);
    $dbfields = array('id');
    $dbvalues = array('null');
    $error = '';

    $sql = "insert into chat_alias ";
    while (list($k, $v) = each($this->form_data)) {
  		$dbfields[] = $k;
			$dbvalues[] = $this->command->db_quote($v);
    }
    $sql .= '(' . join(',', $dbfields) . ') ';
    $sql .= 'values(' . join(',', $dbvalues) . ')';
    $dbh->Execute($sql);
    $dbh->ErrorMsg() ? $error = "Chat alias addition failed -- " . $dbh->ErrorMsg() : '';
    $this->interface_data[error] = $error;
    if ($error != '') { 
      $this->operation = 'add';
      $this->Initialize();
    }
    $template =& $this->theme->template;
    $template->assign('idata', $this->interface_data);
  }
  
  function doEdit() {
    global $dbh;
    global $config;
    $dbfields = array();
    $error = '';

    if ($userinfo[id] != $this->interface_data[uid]) {
      $this->interface_data[error] = 'Illegal access attempt at non-owned character.';
      $this->operation = 'search';
      $this->Initialize();
      $this->theme->template->assign('idata', $this->interface_data);
      return 0;
    }    

    if($this->interface_data[id] == $this->form_data[id]) { // Ensure we are updating ourselves
      $this->form_data[modify_by] = $userinfo[id];
      $this->form_data[modify_date] = $this->command->formatUNIXTimestamp();
      
      $sql = "update chat_alias set ";
      while (list($k,$v) = each ($this->form_data)) {
        $dbfields[] = "$k = " . $this->command->db_quote($v);
      }
      $sql .= join(',', $dbfields);
      $sql .= " where id = ". $this->command->db_quote($this->form_data[id]);
      $dbh->Execute($sql);
      $dbh->ErrorMsg() ? $error = "Chat alias modification failed -- " . $dbh->ErrorMsg() : '';
    } else {
        $error = "Form improperly submitted.";
    }
    $this->interface_data[error] = $error;
    $this->interface_data = array_merge($this->interface_data, $this->command->getInterfaceData($this->search));
    $this->theme->template->assign('idata', $this->interface_data);
    $this->Initialize();
  }
  
  function doDelete() {
    global $dbh;
    $error = '';
    /// Need to set the search key and id in the interface block
    if ($userinfo[id] != $this->interface_data[uid]) {
      $this->interface_data[error] = 'Illegal access attempt at non-owned character.';
      $this->operation = 'search';
      $this->Initialize();
      $this->theme->template->assign('idata', $this->interface_data);
      return 0;
    }    

    if(isset($this->interface_data[id]) ) { // Valid delete ID lookup
      $delete_dt = $this->command->db_quote($this->command->formatUNIXTimestamp());
      $sql = <<<END_SQL
update chat_alias 
set delete_date = $delete_dt
set status = 5
where id = {$this->interface_data[id]}
END_SQL;
      $dbh->Execute($sql);
      $dbh->ErrorMsg() ? $error = "Chat alias delete failed -- " . $dbh->ErrorMsg() : '';
    } else {
      $error = "Invalid block id selected.";
    }
    if ($error = '') { $this->interface_data = array(); }
    $this->interface_data[error] = $error;
  }
  
}

/*
* Interface Initialization
*    The interface class itself is now initialized, the operation variable 
*    used here is always included from the user interface GET or POST request.
*    The Interface constructor creates the global interface variable which 
*    makes it available to the other classes which are initialized by the
*    Initialize method. We cannot initialize in the constructor, as the object
*    is not actually instantiated yet.
*/
$gui = new LocalInterface();

/*    DarkPortal institutes a common set of CGI parameters which should always
*    be used when performing search or lookup operations for user data. The
*    basic method to create a search pattern is handled by three basic lookup
*    variables:
*    - searchby ==> the search lookup table for the query.
*    - searchid ==> the id to use when performing the search.
*    - searchname ==> If the search needs to lookup on an alternate column, 
*      indicate a name for that column.
*    In future releases, this API will be expanded to allow for ranged 
*    searching, and joins on multiple tables. Currently, the search function
*    is implemented directly in the Interface class. This will be changed to
*    be moved to the mainfile.
*/
$gui->Initialize();

/*-----------------------------------------------------------------------------
* Operation Logic
*    Below is a sample of operational logic, which determines what the operation
*    is to be performed, and attempts to perform methods within the GUI to 
*    enact requests by the user. Note that the interface itself will automatically
*    retrieve the appropriate user interface layout blocks from the database 
*    during initialization. User interface methods should never generate output,
*    they should only extract or place data in the database based upon the 
*    user's request.
*/
if (preg_match("/^search/i", $operation)) {
  $gui->search();
} elseif (preg_match("/^submit/i", $operation)) {
  // It is typical to set the form data into the interface class if the operation
  // is a submission of information. Form data fields typically take on the form
  // of form_<dataitem>. This transposition may be modified in future releases.
  isset($form_id) ? $gui->form_data[id] = $form_id : '';
  $gui->form_data[name] = $form_name;
  // If an operation does not require input validation, it is placed first.
  if ($form__operation == 'delete') { 
    $gui->doDelete();
  } elseif ($gui->validateForm()){ 
    // Once the form data is placed, it can be validated. 
    // The validation is interface dependent.
    if ($form__operation == 'edit') {
      $gui->doEdit();
    } elseif ($form__operation == 'add') {
      $gui->doAdd();
    } 
  } else {
    if ($form__operation == 'edit') {
      $gui->edit();
    } elseif ($form__operation == 'add') {
      $gui->add();
    } else {
      $gui->unknownError();
    }
  }
} elseif (preg_match("/^add/", $operation)) {
  $gui->add();
} elseif (preg_match("/^edit/", $operation)) {
  $gui->edit();
} elseif (preg_match("/^delete/", $operation)) {
  $gui->delete();
}

/*-----------------------------------------------------------------------------
* Interface Display
*    Once the operations (if any) are complete, the display method is called.
*    this enacts the object's theme to display its appropriate blocks.
*/
print $gui->display();

?>
