<?php
class PluginMethodFactory extends MethodFactory{
/*------------------------------------------------------------------------
**------------------------------------------------------------------------
*/
  function PluginMethodFactory() {
    $this->MethodFactory();
  }

  function loadSession($sid, $admin = false) {
    global $dbh, $userinfo;
    list($dbt_sess, $scol) = array_values($this->DBGetTable('chat_session'));
    list($dbt_alias, $acol) = array_values($this->DBGetTable('alias'));
    list($dbt_room, $rcol) = array_values($this->DBGetTable('chat_room'));
    
    $sid_q = $this->db_quote($sid);
    $sql = <<<END_SQL
      select $scol[id], $scol[uid], $scol[aid], $acol[name], $rcol[name] as roomname, 
        $scol[rid], $scol[lastread], $scol[firstread], $scol[updated], 
        $acol[entry_message], $acol[exit_message], $scol[status], $scol[status_description]
      from $dbt_sess, $dbt_alias, $dbt_room
      where $scol[id] = $sid_q 
      and  $scol[aid] = $acol[id]
      and  $scol[rid] = $rcol[id]
      order by $scol[created]
END_SQL;
    $result = $dbh->Execute($sql) or trigger_error ("[loadSession] Could not retrieve session data $sql", FATAL);
    if (($admin == false ) && 
        ($userinfo[id] != $result->fields[uid])) {
      return array();
    }
    return $result->fields;
  }

  function loadSessionByAlias($aid, $admin = false) {
    global $dbh, $userinfo;
    list($dbt_sess, $scol) = array_values($this->DBGetTable('chat_session'));
    list($dbt_alias, $acol) = array_values($this->DBGetTable('alias'));
    list($dbt_room, $rcol) = array_values($this->DBGetTable('chat_room'));
    
    $aid_q = $this->db_quote($aid);
    $sql = <<<END_SQL
      select $scol[id], $scol[uid], $scol[aid], $acol[name], $rcol[name] as roomname, 
        $scol[rid], $scol[lastread], $scol[firstread], $scol[updated], 
        $acol[entry_message], $acol[exit_message], $scol[status], $scol[status_description]
      from $dbt_sess, $dbt_alias, $dbt_room
      where $scol[aid] = $aid_q 
      and  $scol[aid] = $acol[id]
      and  $scol[rid] = $rcol[id]
      order by $scol[created]
END_SQL;
    $result = $dbh->Execute($sql) or trigger_error ("[loadSession] Could not retrieve session data $sql", FATAL);
    if (($admin == false ) && 
        ($userinfo[id] != $result->fields[uid])) {
      return array();
    }
    return $result->fields;
  }
  
  function systemSession($session) {
    $session[rid] = (isset($session[rid])) ? $session[rid]: 0;
    $sys_session = array(id => $session[id],
			 uid => 1,
			 aid => 1,
			 rid => $session[rid],
			 name => 'System'
			 );
    return $sys_session;
  }

/*------------------------------------------------------------------------
**------------------------------------------------------------------------
*/
  function chatCleanup($session) {
    global $dbh;
    global $config;

    list($dbt_sess, $scol) = array_values($this->DBGetTable('chat_session'));
    list($dbt_alias, $acol) = array_values($this->DBGetTable('alias'));
    list($dbt_room, $rcol) = array_values($this->DBGetTable('chat_room'));

    $sql = <<<END_SQL
      select $scol[id], $scol[uid], $scol[aid], $scol[rid],
        $scol[updated], $acol[name], $scol[status], $scol[status_description],
        $rcol[name] as roomname 
      from $dbt_sess, $dbt_alias, $dbt_room
        where $scol[status] >= 200
        and $scol[aid] = $acol[id]
        and $scol[rid] = $rcol[id]
END_SQL;
    $result = $dbh->Execute($sql) or trigger_error ("[chatCleanup] Could not find any sessions." . $dbh->ErrorMsg(), ERROR);
    while ($boot_session = $result->fields) {
      $result->MoveNext();
      $this->logMessage($boot_session, $boot_session[status], $boot_session[status_description], 5);
    }
    $sql = <<<END_SQL
      delete from $dbt_sess where $scol[status] >= 200;
END_SQL;
    $dbh->Execute($sql) or trigger_error("[chatCleanup] Failed to delete booted sessions", WARN);
  
    $sql = <<<END_SQL
      select $scol[id], $scol[uid], $scol[aid], $scol[rid],
        $acol[name], $rcol[name] as roomname, $scol[updated]
      from $dbt_sess, $dbt_alias, $dbt_room
      where $scol[aid] = $acol[id]
        and $scol[rid] = $rcol[id]
END_SQL;
    $result = $dbh->Execute($sql) or trigger_error ("[chatCleanup] Could find expired sessions.", FATAL);
    $delete = array();
    while ($tmp_session = $result->fields) {
      $CALC = $this->calculateSeconds($tmp_session[updated], $this->formatUNIXTimestamp());
      if ($CALC > $config[chat][timeout_session]) {
        $this->writeChatMessage($this->systemSession($tmp_session), sprintf($config[chat][message_timeout], $tmp_session[name], $tmp_session[roomname]));
        $this->logMessage($tmp_session, $config[chat][log_session_expire] , 7);
        $to_delete[] = $this->db_quote($tmp_session[id]);
      }
      $result->MoveNext();
    }
    
    $sql = <<<END_SQL
      select $scol[id], $scol[uid], $scol[aid], $scol[rid],
        $acol[name], $rcol[name] as roomname, $scol[created]
      from $dbt_sess, $dbt_alias, $dbt_room
      where $scol[aid] = $acol[id]
        and $scol[rid] = $rcol[id]
        and $scol[rid] = '$session[rid]' 
      order by $scol[created]
END_SQL;
    $result = $dbh->Execute($sql) or trigger_error("[chatCleanup] Could not perform duplicate session lookup.", FATAL) ;

    $flags = array();
    $tmp_session = array();
    while($data = $result->fields) {
      $id = $this->db_quote($data[id]);
      if (isset($flags[$data[aid]][id])) {
      	if ($data[created] < $flags[$data[aid]][dt]) {
      	  $to_delete[] = $flags[$data[aid]][id];
      	} else {
      	  $to_delete[] = $flags[$data[aid]][id];
      	  $tmp_session[] = $data;
      	  $flags[$data[aid]][id] = $id;
      	  $flags[$data[aid]][dt] = $data[created];
      	}
      } else {
      	$flags[$data[aid]][id] = $id;
      	$flags[$data[aid]][dt] = $data[created];
      }
      $result->MoveNext();
    }
    while (list($k, $sess) = each ($tmp_session)) {
      $this->writeChatMessage($this->systemSession($sess), sprintf($config[chat][message_timeout], $sess[name], $sess[roomname]));
      $this->logMessage($sess, $config[chat][log_session_expire] , 7);
    }
    
    if (sizeof($to_delete) > 0) {
      reset($to_delete);
      $sql = "delete from $dbt_sess where $scol[id] in (" . join(", ", ($to_delete)) . ")";
      $result = $dbh->Execute($sql) or trigger_error ("[chatCleanup] Could not remove duplicate/expired sessions.", FATAL);
    }

    return true;
  }

	function buildDynamicRoom($session) {
		global $dbh;

    list($dbt, $dbc) = array_values($this->DBGetTable('chat_room'));
    
		$uid_q = $this->command->db_quote($this->form_data[uid]);
		$created = $this->command->db_quote($this->command->formatUNIXTimestamp());
		$modified = $this->command->db_quote($this->command->formatUNIXTimestamp());
		$used = $this->command->db_quote($this->command->formatUNIXTimestamp());
		$sql = <<<END_SQL
insert into $dbt
($dbc[id], $dbc[uid], $dbc[name], $dbc[short_desc], $dbc[long_desc], $dbc[url], 
  $dbc[occupancy], $dbc[status], $dbc[status_msg], $dbc[privilege], $dbc[visible], 
  $dbc[private], $dbc[create_date], $dbc[modify_date], $dbc[modify_by],
  $dbc[used_date])
values 
(null, $uid_q, $name_q, $desc_q, '', '', $config[chat][dynroom_occupancy], $config[chat][dynroom_status],
'Dynamically created room', $config[chat][dynroom_privilege], $config[chat][dynroom_visible], $config[chat][dynroom_private], $created, $modified,
$uid_q, $used)
END_SQL;
	}

	function cleanSession ($session) {
		global $dbh;
    list($dbt, $dbc) = array_values($this->DBGetTable('chat_session'));

		$id_q = $this->db_quote($session[id]);
		$sql = <<<END_SQL
		delete from $dbt where $dbc[id] = $id_q
END_SQL;
		$result = $dbh->Execute($sql) or trigger_error("[updateSession] Could not update the session.", FATAL);
	}

  function updateSession($session) {
    global $dbh;
    list($dbt, $dbc) = array_values($this->DBGetTable('chat_session'));

    $timestamp = $this->formatUNIXTimestamp();
    $sql = <<<END_SQL
      UPDATE $dbt set
      $dbc[rid] = $session[rid],
      $dbc[lastread] = $session[lastread],
      $dbc[firstread] = $session[firstread],
      $dbc[updated] = '$timestamp'
      where $dbc[id] = '$session[id]'
END_SQL;
    $result = $dbh->Execute($sql) or trigger_error("[updateSession] Could not update the session: $sql", FATAL);
  }

/*------------------------------------------------------------------------
**------------------------------------------------------------------------
*/
  function writeChatMessage($session, $message, $recipients = '') {
    global $dbh, $config;
    list($dbt, $dbc) = array_values($this->DBGetTable('chat_message'));
    
    $message_q = strip_tags($message,'<DIV>');
    $message_q = $this->parseTags($message_q, $config[chat][markup], 1, $config[chat][image]);
    $message_q = $this->db_quote($message_q, true);
    $recipients_q = $this->db_quote($recipients);
    $from_room = $this->db_quote($session[from_room]);
    
    $sql = <<<END_SQL
insert into $dbt
($dbc[id], $dbc[rid], $dbc[aid], $dbc[recipient], $dbc[from_room], $dbc[message],
  $dbc[postdate])
values 
(null,$session[rid], $session[aid], $recipients_q, $from_room,  $message_q, 
  now())
END_SQL;
    $result = $dbh->Execute($sql) or trigger_error("SQL: $sql -- " . $dbh->ErrorMsg(), FATAL);
  }
/*------------------------------------------------------------------------
**------------------------------------------------------------------------
*/
  function logMessage($session, $mid, $detail = '', $severity = 8) {
    global $dbh;

    list($dbt, $dbc) = array_values($this->DBGetTable('chat_log'));
    $module = $this->db_quote(PHP_SELF);
    $sid = $this->db_quote($session[id]);
    $detail = $this->db_quote($detail);
    $timestamp = $this->db_quote($this->formatUNIXTimestamp());

    $sql = <<<END_SQL
      insert into $dbt
      ($dbc[id], $dbc[uid], $dbc[aid], $dbc[rid], $dbc[mid], $dbc[details], 
        $dbc[sid], $dbc[severity], $dbc[module], $dbc[entry_date])
      values (null, $session[uid], $session[aid], $session[rid], $mid, $detail, 
        $sid, $severity, $module, $timestamp)
END_SQL;
    $result = $dbh->Execute($sql) or trigger_error("[logMessage] Could not log message ($mid): $sql");
  }

  function forceLogout($message = '') {
    global $dbh;
    global $sid;
    global $config;
    global $userinfo;
    list($dbt, $dbc) = array_values($this->DBGetTable('chat_log'));

    $sid_q = $this->db_quote($sid);
    $sql = "select $dbc[entry_date], $dbc[mid], $dbc[sid], $dbc[details] from $dbt where $dbc[sid] = $sid_q";
    $result = $dbh->Execute($sql);
    $result->MoveLast();
    if (isset($result->fields[details]) && $result->fields[details] != '') {
    	$message = $result->fields[details];
    }
    $admin_oper = $config[chat][admin_oper][$result->fields[mid]];
    if ($userinfo[chat][form_target] == '_blank') {
      $JSCRIPT = "parent.window.opener.location = \"$config[site_url]/$config[plugin_path]/logout.php?operation=$admin_oper&message=$message\";parent.window.close();";
    } else {
      $JSCRIPT = "top.document.location = \"$config[site_url]/$config[plugin_path]/logout.php?operation=$admin_oper&message=$message\";";
    }
    print <<<END_HTML
<HTML><HEAD>
  <SCRIPT>
    $JSCRIPT
  </SCRIPT>
</HEAD>
<BODY>
</BODY>
</HTML>
END_HTML;
  }
  
}

?>
