<?php
/*-----------------------------------------------------------------------------
** signon.php - DarkConverse chat signon interface
**
** DESCRIPTION
**    
** AUTHOR
**    Fred Hirsch
** COPYRIGHT
**    (C) 2001 Dark Hart Design
**    GNU Public License (see LICENSE file for details)
**-----------------------------------------------------------------------------
*/
/*-----------------------------------------------------------------------------
** Initializing the Interface classes
**    The first step in initialization is to load the core system function and
**    setup file. The "mainfile" loads all the configuration parameters, session
**    handler and database loader for the various sub-elements. Unlike other 
**    paths in the system, the mainfile include must be relative to the location
**    where the current script is running. From the mainfile, the core include
**    path is setup for all the secondary includes.
*/
$plugin = 'chat';

include("../../function.php");
$interface = 'chat.signon';
$interface_title = 'DarkPortal Chat - Signon';

if ($config[chat][guest_entry] == false && empty($userinfo[groups][User])) {
  $reason = "You must be logged to a valid chat access account in to use the chat system.";
  include("$include_path/noaccess.php");
  exit;
}

class LocalInterface extends Interface {

  function LocalInterface() {
    $this->Interface();
  }

  function initializeLocal() {
  }

  function main(&$theme) {
    global $dbh;
    global $config; 
    global $userinfo;
    
    $theme->assign('userinfo', $userinfo);
    $theme->assign('idata', $this->interface_data);
  }

  function buildChatSession () {
	  global $userinfo;
	  global $dbh;

    list($dbt_mess, $mcol) = array_values($this->command->DBGetTable('chat_message'));
    list($dbt_sess, $scol) = array_values($this->command->DBGetTable('chat_session'));

    if (empty($this->form_data[aid])) {
      $this->interface_data[error] = 'No alias ID selected. Create an alias or character to enter the chat.';
      return false;
    }
    if ($this->form_data[rid] == '' && $this->form_data[dyn_room] == '') {
      $this->interface_data[error] = 'No room selected. Create a room or select a pre-existing room to enter.';
      return false;
    } elseif ($gui->form_data[dyn_room] != '') {
			$this->form_data[rid] = $this->buildDynamicRoom();
		}

    
		$sess = explode(" ", microtime());
		$this->chat_session[id] = (sprintf ("A%06d.%09d", $sess[0] * 1000000, $sess[1]));
		$this->chat_session[uid] = $this->form_data[uid];
		$this->chat_session[aid] = $this->form_data[aid];
		$this->chat_session[rid] = $this->form_data[rid];
		$this->chat_session[updated] = $this->command->db_quote($this->command->formatUNIXTimestamp());
		$sql = <<<END_SQL
			   select $mcol[id] from $dbt_mess where $mcol[rid] = {$this->chat_session[rid]} order by $mcol[id] desc
END_SQL;
		$result = $dbh->SelectLimit($sql, $userinfo[chat][message_max]) or trigger_error("Error: $sql ". $dbh->ErrorMsg(), FATAL);
		$result->MoveLast();
 		if ($result->fields[id] > 0) {
 			$this->chat_session[firstread] = $result->fields[id];
 			$this->chat_session[lastread] = $result->fields[id];
 		} else {
 			$this->chat_session[firstread] = 0;
 			$this->chat_session[lastread] = 0;
 		}
    $this->chat_session[ip_address] = $this->command->db_quote(getenv('REMOTE_ADDR'));

		$sql = <<<END_SQL
insert into $dbt_sess
	($scol[id], $scol[uid], $scol[aid], $scol[rid], $scol[firstread], $scol[lastread],
	  $scol[created], $scol[updated], $scol[ip_address])
	values('{$this->chat_session[id]}',
      {$this->chat_session[uid]},
      {$this->chat_session[aid]},
      {$this->chat_session[rid]},
	  	{$this->chat_session[firstread]},
	  	{$this->chat_session[lastread]},
      {$this->chat_session[updated]},
	  	{$this->chat_session[updated]},
	  	{$this->chat_session[ip_address]})
END_SQL;
    $dbh->Execute($sql) or trigger_error ("SQL: $sql -- " . $dbh->ErrorMsg(), FATAL);

		$this->chat_session = $this->command->loadSession($this->chat_session[id]);
		$this->command->chatCleanup($this->chat_session);

		if ($this->validateLogin()) {
			return true;
		} else {
			$this->command->cleanSession($this->chat_session);
			return false;
		}
	}

  function validateLogin() {
    global $dbh;
		global $userinfo;
		global $config;
	
	  // CHECK: Delete expired bans.
    $this->command->checkUserBan($this->plugin);
    
    //CHECK: Make sure UID is valid.
		if ($this->form_data[uid] != $userinfo[id]) {
			$this->command->logMessage($this->command->systemSession(), $config[chat][error_uid_invalid], '$error', 5);
			trigger_error ("User ID invalid, do not attempt to hijaak this form.", FATAL);
		}
    
    //CHECK: Make sure the alias id the user is using belongs to them.
    //CHECK: See if the alias is anonymous, and if anonymous id's are allowed.
    list($dbt_alias, $acol) = array_values($this->command->DBGetTable('alias'));
    list($dbt_alst, $ascol) = array_values($this->command->DBGetTable('alias_status'));
    list($dbt_alac, $aacol) = array_values($this->command->DBGetTable('alias_activity'));

    $aid_q = $this->command->db_quote($this->form_data[aid]);
    $sql = <<<END_SQL
		select $acol[id], $acol[uid], $acol[status], $acol[activity], 
		  $ascol[name] as st_name, $ascol[description] as st_desc, 
		  $aacol[description] as act_desc, $aacol[name] as act_name
		from $dbt_alias, $dbt_alst, $dbt_alac
		where $acol[activity] = $aacol[id]
		  and $acol[status] = $ascol[id]
		  and $acol[id] = $aid_q
END_SQL;
		$result = $dbh->Execute($sql) or trigger_error('Cannot validate alias information', FATAL);

	  if (!$config[chat][anon_login] && $result->fields[id] == $config[anon_login_id]) {
	    $error .= "Anonymous logins are disabled on this chat. Please select a valid alias and retry.";
			$this->command->logMessage($this->command->systemSession(), $config[chat][error_aid_invalid], $error, 5);
	  } elseif ($result->fields[uid] != $userinfo[id] && $result->fields[id] != $config[chat][anon_login_id]) {
			$error .= "The alias selected does not belong to your account\n";
			$this->command->logMessage($this->command->systemSession(), $config[chat][error_aid_invalid], $error, 5);
	  } elseif (empty($result->fields[id]) || $result->fields[id] == '') {
			$error .= "The alias you are attempting to use does not exist.\n";
			$this->command->logMessage($this->command->systemSession(), $config[chat][error_aid_invalid], $error, 5);
		}

    if ($result->fields[activity] > 1) {
      $error .= "This character ID has been set {$result->fields[act_name]}: {$result->fields[act_desc]}";
    }
    //CHECK: Make sure user is allowed to enter room.
    list($dbt_room, $rcol) = array_values($this->command->DBGetTable('chat_room'));
		$rid_q = $this->command->db_quote($this->form_data[rid]);
		$sql = <<<END_SQL
		select $rcol[name], $rcol[uid], $rcol[occupancy], $rcol[status], 
		  $rcol[status_msg], $rcol[privilege], $rcol[visible], $rcol[private]
	  from $dbt_room 
	  where $rcol[id] = $rid_q
END_SQL;
		$result = $dbh->Execute($sql) or trigger_error('Cannot validate room information.', FATAL);;
		$room_info = $result->fields;

    list($dbt_sess, $scol) = array_values($this->command->DBGetTable('chat_session'));
    $sql = <<<END_SQL
		 select count($scol[id]) as room_count 
		 from $dbt_sess 
		 where $scol[rid] = $rid_q
END_SQL;
		$result = $dbh->Execute($sql) or trigger_error('Cannot chat occupancy levels.', FATAL);;
		$sess_info = $result->fields;
		// First, check if the room is private and if the user has access to the room
		if ($room_info[private] == 1 && 
		    $userinfo[chat][enter_private] != 1 &&
		    $userinfo[id] != $room_info[uid]) {
      list($dbt_rmac, $racol) = array_values($this->command->DBGetTable('chat_room_access'));
			$sql = <<<END_SQL
			select $racol[rid]
			from $dbt_rmac
			where $racol[rid] = $rid_q 
			  and $racol[aid] = $aid_q
END_SQL;
			
			$result = $dbh->Execute($sql) or trigger_error('Cannot validate room access.', FATAL);;
			if (empty($result->fields[rid]) || $result->fields[rid] == '') {
				$error .= "Room $room_info[name] is a private room. You do not have permission to enter.\n";
			}
		} 
	    // Check if the room is disabled
		if ($room_info[status] == 2) {
			$error .= "Room $room_info[name] is currently closed. \nReason: $room_info[status_msg]";
		}
		// Next, check if the room is full
		if ($sess_info[room_count] > $room_info[occupancy] && $userinfo[chat][enter_full] != 1) {
			$error .= "Room $room_info[name] is currently full. Try another room or wait until a user leaves.\n";
		}
		// Last, check if the site is full
		$sql = <<<END_SQL
			 select count($scol[id]) as room_count 
			 from $dbt_sess
END_SQL;
		$result = $dbh->Execute($sql) or trigger_error('Cannot validate room occupancy.', FATAL);;
		$sess_info = $result->fields;
		if ($sess_info[room_count] > $config[chat][max_occupancy]) {
			$error .= "The chat is currently full, please wait until another user leaves.\n";
		}
	    //CHECK: Restrict the number of logins available to the user.
		$uid_q = $this->command->db_quote($this->form_data[uid]);
		$sql = <<<END_SQL
			 select $scol[rid], count($scol[id]) as user_count 
			 from $dbt_sess
			 where $scol[uid] = $uid_q 
			 group by $scol[rid]
END_SQL;
		$result = $dbh->Execute($sql) or trigger_error('Cannot validate login restrictions', FATAL);;
		#print "### {$result->fields[user_count]}";
		if ($result->fields[rid] != '') {
			while ($sess_info = $result->fields) {
				$total_count += $sess_info[user_count];
				if ($sess_info[rid] == $this->form_data[rid] && $sess_info[user_count] > $userinfo[chat][login_perroom]) {
					($userinfo[chat][login_perroom] > 1) ? $plural = 's' : $plural = '';
					$error .= "You have already signed into $room_info[name], {$userinfo[chat][login_perroom]} time$plural. Logout one of the existing characters to proceed.\n";
				}
				$result->MoveNext();
			//print "###$total_count .. $sess_info[user_count]:$sess_info[rid] .. {$this->form_data[rid]}:$error<br>";
			}
			if ($total_count > $userinfo[chat][login_perchat]) {
				($userinfo[chat][login_perchat] > 1) ? $plural = 's' : $plural = '';
				$error .= "You have already signed into the chat, $userinfo[login_perchat] time$plural. Logout one of the existing characters to proceed.\n";
			}
		}
		
		$error = nl2br($error);
		$this->interface_data[error] = $error;
		$this->theme->assign('idata', $gui->interface_data);
	  if ($error == '') { return true; } else { return $false; }
  }

  function buildChatInterface () {
    global $config;

    $sessid = $this->chat_session[id];
    $output = <<<END_HTML
<HTML><HEAD><TITLE>Chat Frame Mockup</TITLE>
  <script language="JavaScript"><!--
		 function shutdown() {
			 top.document.location = "http://192.168.66.166/portal/plugins/chat/logout.php";
		 }
  //--></script>
  </HEAD>
  <LINK REL=stylesheet HREF="$config[site_url]/plugins/chat/chat.css" TYPE="text/css">
  <FRAMESET COLS=*,200 BORDER=1 FRAMEBORDER=1 >
    <FRAMESET ROWS=75,*,125 BORDER=1 FRAMEBORDER=1 >
      <FRAME NAME="roominfo" SRC="roominfo.php?sid=$sessid" SCROLLING=NO MARGINHEIGHT=0 MARGINWIDTH=0>
      <FRAME NAME="message" SRC="message.php?sid=$sessid" SCROLLING=AUTO MARGINHEIGHT=0 MARGINWIDTH=0>
      <FRAMESET COLS=300,* BORDER=1 FRAMEBORDER=1 >
        <FRAME NAME="submit" SRC="submit.php?sid=$sessid" SCROLLING=AUTO MARGINHEIGHT=0 MARGINWIDTH=0>
        <FRAME NAME="plugin" SRC="plugin.php?sid=$sessid" SCROLLING=NO MARGINHEIGHT=0>
      </FRAMESET>
    </FRAMESET>
    <FRAMESET ROWS=*,100,1 BORDER=1 FRAMEBORDER=0>
      <FRAME NAME="userlist" SRC="userlist.php?sid=$sessid" SCROLLING=AUTO MARGINHEIGHT=0 MARGINWIDTH=0>
      <FRAME NAME="elevator" SRC="blank.php" SCROLLING=NO MARGINHEIGHT=0 MARGINWIDTH=0>
      <FRAME NAME="code" SRC="status.php?sid=$sessid" SCROLLING=NO MARGINHEIGHT=0 MARGINWIDTH=0>
    </FRAMESET>
  </FRAMESET>
</HTML>
END_HTML;
    return $output;
  }
  
  function updateAliasLogin() {
  	global $dbh;
  	$aid_q = $this->command->db_quote($this->form_data[aid]);
  	$login_dt_q = $this->command->db_quote($this->command->formatUNIXTimestamp());
    list($dbt_alias, $acol) = array_values($this->command->DBGetTable('alias'));
		$sql = <<<END_SQL
	update $dbt_alias 
	set $acol[login_date] = $login_dt_q, 
	  $acol[login_count] = $acol[login_count] + 1 
	  where $acol[id] = $aid_q
END_SQL;
		$dbh->Execute($sql);
		if ($dbh->ErrorMsg()) { trigger_error("SQL: $sql -- " . $dbh->ErrorMsg(), ERROR); }
	}  	
}

$gui = new LocalInterface();
$gui->Initialize();
if (preg_match("/^login/i", $operation)) {
	if ($gui->buildChatSession()) {
		//$gui->chat_session = $gui->command->loadSession($gui->chat_session[id]);
		$gui->command->writeChatMessage($gui->command->systemSession($gui->chat_session), sprintf($config[chat][message_enter],$gui->chat_session[name],$gui->chat_session[roomname],$gui->chat_session[entry_message]));
		$gui->command->logMessage($gui->chat_session, $config[chat][log_session_enter]);
		$gui->updateAliasLogin();
		print $gui->buildChatInterface($sessid);
		exit;
	} else {
		unset($gui->operation);
		$gui->Initialize();
		$gui->theme->assign('idata', $gui->interface_data);
	}
}
print $gui->display();


?>
