<?php
/*
* User Alias Interface
*
* This interface allows the adding, deleting and modification of user
* aliases. 
*
* @version  0.2.0 beta
* @author   Fred Hirsch
* @copyright  Dark Hart Design  2001
* @package API
*/

/**
* Initializing the Interface classes
*    The first step in initialization is to load the core system function and
*    setup file. The "mainfile" loads all the configuration parameters, session
*    handler and database loader for the various sub-elements. Unlike other 
*    paths in the system, the mainfile include must be relative to the location
*    where the current script is running. From the mainfile, the core include
*    path is setup for all the secondary includes.
*/
$plugin = 'user';    // Plugin Name
include_once("../../function.php");
/*     The next variables set the interface name and the plugin name of the 
*     current script. The interface name describes shared functionality within
*     a specific plugin, while the plugin name is used to determine the 
*     location of helper scripts to the interfaces.
*/
$interface = 'user.alias'; 
$interface_title = 'DarkPortal - User Alias Manager';
$interface_cols = array();

/*-----------------------------------------------------------------------------
* Authentication Handling
*    In the case that a specific interface should limit access to its functions,
*    the following code sample can be used to limit access and provide the user
*    with a reason for the denial. This sample could be expanded to force the
*    user to be a member of specific group(s) or use a plugin config variable
*    to turn on group level authentication.
*/
if ($config[chat][guest_entry] == false && empty($userinfo[groups][User])) {
  $reason = "You must be logged to a valid access account in to use the alias system.";
  include("$include_path/noaccess.php");
  exit;
}

isset ($searchby) ? $operation .= ".$searchby" : '';
isset ($form__operation) ? $operation .= ".$form__operation" : '';

/*-----------------------------------------------------------------------------
* >>>>>>>>>>>>>>>>>>>>>>>>>>  END OF GLOBAL CODE <<<<<<<<<<<<<<<<<<<<<<<<<<<<<
*/

/**
* API Interface 
*
* The following template provides the basic level of functionality for all
* Dark Portal plugin interfaces. It serves as a common interface or API to
* perform all display functions for Dark Portal addons and current core
* functionality. For detailed developer instructions, refer to comments in
* this file, and the developer manual included with Dark Portal.
* @version  0.2.0 beta
* @author   Fred Hirsch
* @copyright  Dark Hart Design  2001
* @package API
*/
class LocalInterface extends Interface {

/**
* API Interface Constructor
*
* The local interface contructor should usually only call the super class
* constructor. In some cases, it may be useful to add some parameters such
* as static theme ID (admin interfaces) and so forth.
*
* @return VOID
*/
  function LocalInterface() {
    $this->Interface();
  }

  function initializeLocal() {
    global $dbh;
    global $userinfo;

      if (isset($this->interface_data[id]) && $this->interface_data[id] != '') {
        list($dbtable, $acol) = array_values($this->command->DBGetTable('alias'));
              
        while (list($k, $v) = each($this->interface_data)) {
            if (is_array($v) || $k = 'error') { continue; }
            $this->interface_data[no_parse][$k] = $v;
            $this->interface_data[$k] = (isset($v) && $v != '') ? $this->command->parseTags($v,1,0,0) : 'N/A';
        }

        $sql = "select name, value from alias_extra where aid = {$this->interface_data[id]}";
        $result = $dbh->Execute($sql);
        $dbh->ErrorMsg() ? $this->interface_data[error] =  "Cannot find extra information for {$this->interface_data[name]}." : '';
        
        while ($adata = $result->fields) {
          $result->MoveNext();
          if (preg_match("/^\d{14}$/", $adata[value])) { $adata[value] = $this->command->formatDBTimestamp($adata[value]); }
          $this->interface_data[no_parse][extra][$adata[name]] = $adata[value];
          $this->interface_data[extra][$adata[name]] = (isset($adata[value]) && $adata[value] != '') ? $this->command->parseTags($adata[value],1,0,0) : 'N/A';
        }
      
        $this->interface_data[system_d] = $this->command->getNameFromID($this->interface_data[system], 'alias_system');
        $this->interface_data[status_d] = $this->command->getNameFromID($this->interface_data[status], 'alias_status');
        $this->interface_data[activity_d] = $this->command->getNameFromID($this->interface_data[activity], 'alias_activity');
      } elseif (isset($this->interface_data[0][id]) && $this->interface_data[0][id] != '') {
        for ($i = 0; $i < sizeof ($this->interface_data) ; $i++) {
          if (is_array($this->interface_data[$i])) {
            $this->interface_data[$i][system_d] = $this->command->getNameFromID($this->interface_data[$i][system], 'alias_system');
            $this->interface_data[$i][status_d] = $this->command->getNameFromID($this->interface_data[$i][status], 'chat_alias_status');
            $this->interface_data[$i][activity_d] = $this->command->getNameFromID($this->interface_data[$i][activity], 'chat_alias_activity');
          }
        }
      }
      $this->theme->assign('idata', $this->interface_data);
  }
  
/**
* Main Zone Callback Method
*
* @param ObjectRef $theme A reference to the template object to use for assigning template date, etc.
* @return VOID
*/
  function main(&$theme) {
    $theme->assign('idata', $this->interface_data);
  }

  function doAdd() {
    global $dbh;
    global $userinfo;
    
    $this->form_data[uid] = $userinfo[id];
    $this->form_data[can_modify] = 1;
    $this->form_data[create_date] = $this->command->formatUNIXTimestamp();
    $this->form_data[modify_by] = $userinfo[id];
    $this->form_data[modify_date] = $this->command->formatUNIXTimestamp();
    $this->form_data[login_date] = '00000000000000';
    $this->form_data[delete_date] = '00000000000000';
    $this->form_data[inactive_date] = '00000000000000';
    unset ($this->form_data[id]);
    $dbfields = array('id');
    $dbvalues = array('null');
    $error = '';

    $sql = "insert into alias ";
    while (list($k, $v) = each($this->form_data)) {
  		$dbfields[] = $k;
			$dbvalues[] = $this->command->db_quote($v);
    }
    $sql .= '(' . join(',', $dbfields) . ') ';
    $sql .= 'values(' . join(',', $dbvalues) . ')';
    $dbh->Execute($sql);
    $dbh->ErrorMsg() ? $error = "Chat alias addition failed -- " . $dbh->ErrorMsg() : '';
    $this->interface_data[error] = $error;
    if ($error != '') { 
      $this->operation = 'add';
      $this->Initialize();
    }
    $theme =& $this->theme;
    $theme->assign('idata', $this->interface_data);
  }
  
  function edit_extra() {
    global $userinfo;
    $this->interface_data[uid] = $userinfo[id];
  }
  
  function doEdit() {
    global $dbh;
    global $config;
    $dbfields = array();
    $error = '';

    if($this->interface_data[id] == $this->form_data[id] &&
      $this->interface_data[uid] == $userinfo[id]) { // Ensure we are updating ourselves
      $this->interface_data[error] = 'Illegal access attempt at non-owned character.';
      $this->operation = 'search';
      $this->Initialize();
      $this->theme->assign('idata', $this->interface_data);
      return 0;
    }    

    if($this->interface_data[id] == $this->form_data[id]) { // Ensure we are updating ourselves
      $this->form_data[modify_by] = $userinfo[id];
      $this->form_data[modify_date] = $this->command->formatUNIXTimestamp();
      $aid_q = $this->command->db_quote($this->form_data[id]);
      
      $sql = "update alias set ";
      while (list($k,$v) = each ($this->form_data)) {
        $dbfields[] = "$k = " . $this->command->db_quote($v);
      }
      $sql .= join(',', $dbfields);
      $sql .= " where id = ". $this->command->db_quote($this->form_data[id]);

      $dbh->Execute($sql);
      $dbh->ErrorMsg() ? $error = "Chat alias modification failed -- " . $dbh->ErrorMsg() : '';
    } else {
        $error = "Form improperly submitted.";
    }
    $this->interface_data[error] = $error;
    $this->interface_data = array_merge($this->interface_data, $this->command->getInterfaceData($this->search));
    $this->theme->assign('idata', $this->interface_data);
    $this->Initialize();
  }
  function doEditExtra() {
    global $config, $dbh, $userinfo;

    if($this->interface_data[id] == $this->form_data[id] &&
      $this->interface_data[uid] != $userinfo[id]) { // Ensure we are updating ourselves
      $this->interface_data[error] = 'Illegal access attempt at non-owned character.';
      $this->operation = 'search';
      $this->Initialize();
      $this->theme->assign('idata', $this->interface_data);
      return 0;
    }    

    if($this->interface_data[id] == $this->form_data[id]) { // Ensure we are updating ourselves
      $this->form_data[modify_by] = $userinfo[id];
      $this->form_data[modify_date] = $this->command->formatUNIXTimestamp();
      $aid_q = $this->command->db_quote($this->form_data[id]);


      list ($dbt, $dbc) = array_values($this->command->DBGetTable('alias_extra'));
      while (list($k, $v) = each ($this->form_data)) {
        if (substr($k, 0, 3) == 'ex_') {
          $val_q = $this->command->db_quote($v);
          $nam_q = $this->command->db_quote(substr($k,3));
          $nam = substr($k,3);
          if (isset($this->interface_data[extra][$nam])) {
            $sql = "update $dbt set $dbc[value] = $val_q where $dbc[name] = $nam_q and $dbc[aid] = $aid_q";
          } else {
            $sql = "insert into $dbt ($dbc[id], $dbc[aid], $dbc[name], $dbc[value], $dbc[permission]) values (null, $aid_q, $nam_q, $val_q, 0)";
          }
          $dbh->Execute($sql);
          $dbh->ErrorMsg() ? $error = "Chat alias modification failed -- " . $dbh->ErrorMsg() : '';
        }
      }
    } else {
        $error = "Form improperly submitted.";
    }
    $this->interface_data[error] = $error;
    $this->interface_data = array_merge($this->interface_data, $this->command->getInterfaceData($this->search));
    $this->theme->assign('idata', $this->interface_data);
    $this->Initialize();
  }  
  
  function doDelete() {
    global $dbh, $userinfo;
    $error = '';
    /// Need to set the search key and id in the interface block
    if($this->interface_data[id] == $this->form_data[id] &&
       $this->interface_data[uid] != $userinfo[id]) { // Ensure we are updating ourselves
      $this->interface_data[error] = 'Illegal access attempt at non-owned character.';
      $this->operation = 'search';
      $this->Initialize();
      $this->theme->assign('idata', $this->interface_data);
      return 0;
    }    

    if(isset($this->interface_data[id]) ) { // Valid delete ID lookup
      $delete_dt = $this->command->db_quote($this->command->formatUNIXTimestamp());
      $sql = <<<END_SQL
update alias 
set delete_date = $delete_dt,
status = 5
where id = {$this->interface_data[id]}
END_SQL;
      $dbh->Execute($sql) or trigger_error( "Chat alias delete failed -- " . $dbh->ErrorMsg(), FATAL);
    } else {
      $error = "Invalid block id selected.";
    }
    if ($error = '') { $this->interface_data = array(); }
    $this->interface_data[error] = $error;
    $this->operation = 'search';
    unset($this->search);
    $this->Initialize();
  }
  
}

/*
* Interface Initialization
*    The interface class itself is now initialized, the operation variable 
*    used here is always included from the user interface GET or POST request.
*    The Interface constructor creates the global interface variable which 
*    makes it available to the other classes which are initialized by the
*    Initialize method. We cannot initialize in the constructor, as the object
*    is not actually instantiated yet.
*/
$gui = new LocalInterface();

/*    DarkPortal institutes a common set of CGI parameters which should always
*    be used when performing search or lookup operations for user data. The
*    basic method to create a search pattern is handled by three basic lookup
*    variables:
*    - searchby ==> the search lookup table for the query.
*    - searchid ==> the id to use when performing the search.
*    - searchname ==> If the search needs to lookup on an alternate column, 
*      indicate a name for that column.
*    In future releases, this API will be expanded to allow for ranged 
*    searching, and joins on multiple tables. Currently, the search function
*    is implemented directly in the Interface class. This will be changed to
*    be moved to the mainfile.
*/
$gui->Initialize();

/*-----------------------------------------------------------------------------
* Operation Logic
*    Below is a sample of operational logic, which determines what the operation
*    is to be performed, and attempts to perform methods within the GUI to 
*    enact requests by the user. Note that the interface itself will automatically
*    retrieve the appropriate user interface layout blocks from the database 
*    during initialization. User interface methods should never generate output,
*    they should only extract or place data in the database based upon the 
*    user's request.
*/
if (preg_match("/^search/i", $operation)) {
  $gui->search();
} elseif (preg_match("/^submit/i", $operation)) {
  // It is typical to set the form data into the interface class if the operation
  // is a submission of information. Form data fields typically take on the form
  // of form_<dataitem>. This transposition may be modified in future releases.
  isset($form_id) ? $gui->form_data[id] = $form_id : '';
  $gui->form_data[name] = $form_name;
  // If an operation does not require input validation, it is placed first.
  if ($form__operation == 'delete') { 
    $gui->doDelete();
  } elseif ($gui->validateForm()){ 
    // Once the form data is placed, it can be validated. 
    // The validation is interface dependent.
    if ($form__operation == 'edit') {
      $gui->doEdit();
    } elseif ($form__operation == 'edit_extra') {
      $gui->doEditExtra();
    } elseif ($form__operation == 'add') {
      $gui->doAdd();
    } 
  } else {
    if ($form__operation == 'edit') {
      $gui->edit();
    } elseif ($form__operation == 'add') {
      $gui->add();
    } else {
      $gui->unknownError();
    }
  }
} elseif (preg_match("/^add/", $operation)) {
  $gui->add();
} elseif (preg_match("/^edit_extra/", $operation)) {
  $gui->edit_extra();
} elseif (preg_match("/^edit/", $operation)) {
  $gui->edit();
} elseif (preg_match("/^delete/", $operation)) {
  $gui->delete();
}

/*-----------------------------------------------------------------------------
* Interface Display
*    Once the operations (if any) are complete, the display method is called.
*    this enacts the object's theme to display its appropriate blocks.
*/
print $gui->display();

?>
