<?php
/*-----------------------------------------------------------------------------
** user.php - User information manager
**
** DESCRIPTION
**
** AUTHOR
**    Fred Hirsch
** COPYRIGHT
**    (C) 2001 Dark Hart Design
**    GNU Public License (see LICENSE file for details)
**-----------------------------------------------------------------------------
*/
/*-----------------------------------------------------------------------------
** Initializing the Interface classes
**    The first step in initialization is to load the core system function and
**    setup file. The "mainfile" loads all the configuration parameters, session
**    handler and database loader for the various sub-elements. Unlike other 
**    paths in the system, the mainfile include must be relative to the location
**    where the current script is running. From the mainfile, the core include
**    path is setup for all the secondary includes.
*/
include_once("../../function.php");

/*     The next variables set the interface name and the plugin name of the 
**     current script. The interface name describes shared functionality within
**     a specific plugin, while the plugin name is used to determine the 
**     location of helper scripts to the interfaces.
*/
$interface = 'user'; 
$interface_title = 'DarkPortal - User Information';
$plugin = 'user';    // articles

/*     We next load plugin specific functions and configuration. Note that these
**     items follow a specific path pattern which must be used to help 
**     standardize the layout of the plugin directories. One exception to this
**     rule is the use of the core admin tools which use their own path pattern.
*/
include("$include_path/plugins/$plugin/function.php");
include("$include_path/plugins/$plugin/config.php");

/*-----------------------------------------------------------------------------
** >>>>>>>>>>>>>>>>>>>>>>>>>>  END OF GLOBAL CODE <<<<<<<<<<<<<<<<<<<<<<<<<<<<<
*/

/*-----------------------------------------------------------------------------
** CLASS: Interface
**-----------------------------------------------------------------------------
**    The interface class contains all the methods required to handle user 
**    input from the web interface and handles any specialized handling of the
**    display for these requests. This class must always contain a constructor,
**    display(), and main() methods.
*/
class LocalInterface extends Interface{
	
	function LocalInterface() {
		$this->Interface();
	}

	/*-----------------------------------------------------------------------------
	** METHOD: void main(reference template)
	**-----------------------------------------------------------------------------
	** Almost all interfaces should contain a main() method. This will allow the 
	** system to override the parsing of main inside the template and perform the
	** lookup of data which will later be needed to be placed inside the primary
	** zone of the interface. It is also common to create helper and indexing 
	** content in other zones. These may be further aided by adding content methods
	** with the same name as the zone. As an example: It is common to provide 
	** help features to new users in the rightblock zone of the default theme. If
	** customized data is needed in these blocks, it could be done in a rightblock()
	** method.
	*/
	function main(&$theme) {
		// $dbh is the global database connector object and allows abstract database
		// access and queries. This is included in many cases.
		global $dbh;
		// $config contains the system global configuration array as well as any 
		// special configuration options for the plugin. These are stored as a
		// subarray to config ==> $config[<plugin name>][<element>]
		global $config; 
		// $userinfo contains an array from the user's session information of all
		// information, access control and group ownership assigned to the user. 
		// Base level information is stored by associative keys, while access 
		// control is stored in subarrays. As an example, to test for a user
		// belonging to group "Admin", the following key in the array:
		//     $userinfo[groups][Admin] <<== would be set true or 1.
		global $userinfo;
		
		// Note the use of references in passing the template around the interface.
		// This is a crucial performance issue, and should always be used in this
		// form. The template can still be modified directly. It is also important
		// to note that any method which overrides the template class to display
		// a specialized version of blocks must have a template object passed to it.
		// otherwise, the receiving method will not have any reference to build 
		// data into the template.
		$this->template =& $theme;
		$interface = $this->interface;
		// Assigning data to the template must be done whenever new data has been 
		// loaded, or data is removed, edited or added to the database.
		$theme->assign('idata', $this->interface_data);
		// Some blocks are more useful to be added in a repetitive manner. This is
		// especially true when showing summaries of news or other article managers.
		// Performing this operation on the template takes the current block data
		// and applies it to the current main block definition and adds the parsed 
		// block to the template's already parsed information.
		// $addParsedBlock();
	}
	
	function add() {
		global $userinfo;
		// Check if the user is logged in, if so, don't show the add form, show the edit!
		if (isset($userinfo[username]) && $userinfo[username] != '') {
			$this->operation = 'edit';
			$this->edit();
		}
	}

	function doAdd() {
		global $dbh;
		global $userinfo;
		global $config;
		
		unset ($this->form_data[id]);
		$dbfields = array('id');
		$dbvalues = array('null');
		$error = '';
		
		$sql = 'insert into users ';
		while (list($k, $v) = each($this->form_data)) {
			$dbfields[] = $k;
			if ($k != 'password') {
				$dbvalues[] = $this->command->db_quote($v);
			} else {
				$dbvalues[] = "password( " . $this->command->db_quote($v) . ")";
			}
		}
		$dbfields[] = 'create_date';
		$dbfields[] = 'modify_date';
		$dbfields[] = 'modify_by';
		$dbvalues[] = 'now()';
		$dbvalues[] = 'now()';
		$dbvalues[] = 1;

		$sql .= '(' . join(',', $dbfields) . ') ';
		$sql .= 'values(' . join(',', $dbvalues) . ')';
		$dbh->Execute($sql);
		$dbh->ErrorMsg() ? $error = "User addition failed -- " . $dbh->ErrorMsg() : '';

		$user_q = $this->command->db_quote($this->form_data[username]);
		$sql = "select id from users where username = $user_q";
		$result = $dbh->Execute($sql);

    $id = $result->fields[id];
		$id_q = $this->command->db_quote($id);
		// Add default values for groups.
		$sql = "insert into user_groups (uid, gid) values ($id_q, $config[default_user_group]) ";
		$result = $dbh->Execute($sql);
		$dbh->ErrorMsg() ? $error = "User group addition failed -- " . $dbh->ErrorMsg() : '';

    // The system root account will hold all default preference values for the 
    // users to be added.
    $sql = <<<END_SQL
    select name, value, plugin from user_preference where id = 1
END_SQL;
    $result = $dbh->Execute($sql);
    while ($pref_data = $result->fields) {
      $name_q = $this->command->db_quote($pref_data[name]);
      $value_q = $this->command->db_quote($pref_data[value]);
      $plugin_q = $this->command->db_quote($pref_data[plugin]);
  		$sql = "insert into user_preference (id, name, value, plugin) values ($id_q, $name_q, $value_q, $plugin_q)";
  		$dbh->Execute($sql);
  		$dbh->ErrorMsg() ? $error = "User preference addition failed -- " . $dbh->ErrorMsg() : '';
      $result->MoveNext();
    }

		// We won't add to blocks until later.

    if ($error == '') {
      $search[cols] = array('id', 'username', 'password', 'name', 'last_ip', 'login_count', 'login_date');
      $search[key] = 'users';
      $search[id] = $id;
      $search[name] = 'id';

      $userinfo = $this->command->getInterfaceData($search);
      $sql = "select g.id, g.name from groups g, user_groups ug where ug.gid = g.id and ug.uid = $userinfo[id]";
      $result = $dbh->Execute($sql);
      $dbh->ErrorMsg() ? $error .= "Authentication failed -- Could not load group data for user." : '';
      while(list($id, $name) = $result->fields) {
      	$userinfo[groups]['default'] = 0;
      	$userinfo[groups][$name] = $id;
      	$result->MoveNext();
      }
      session_register("userinfo") or die("Could not register session info");
      $this->Initialize();
    } else {
      // User add was not successful, so we should manually delete what we just 
      // added.
      if (isset($id) && $id != '') {
        $sql = "delete from users where id = $id_q";
        $dbh->Execute($sql);
        $sql = "delete from user_groups where id = $id_q";
        $dbh->Execute($sql);
        $sql = "delete from user_preference where id = $id_q";
        $dbh->Execute($sql);
      }        
      $this->interface_data[error] = $error;
      $theme =& $this->theme;
      $theme->assign('idata', $this->interface_data);
    }
	}

	function doEdit() {
		global $dbh;
		global $userinfo;
		
		$dbfields = array();
		$error = '';
		if ($this->interface_data[id] == $this->form_data[id]) {
			$sql = "update users set ";
			while (list($k, $v) = each($this->form_data)) {
				if ($k != 'password') {
					$dbfields[] = "$k = " . $this->command->db_quote($v);
				} else {
					$dbfields[] = "$k = password( " . $this->command->db_quote($v) . ")";
				}
			}
  		$dbfields[modify_date] = 'modify_date = now()';
  		$dbfields[modify_by] = "modify_by = $userinfo[id]";

			$sql .= join(',', $dbfields);
			$sql .= " where id = " . $this->command->db_quote($this->form_data[id]);
			$dbh->Execute($sql);
			$dbh->ErrorMsg() ? $error = "User modification failed -- Database could not fulfil your request" : '';
		} else {
			$error = "Form improperly submitted.";
		}
		$this->interface_data[error] = $error;
		$theme =& $this->theme;
		$this->interface_data = array_merge($this->interface_data, $this->command->getInterfaceData($this->search));
		$theme->assign('idata', $this->interface_data);
	}

	function validateFormLocal() {
		// Email validation
		if (isset($this->form_data[email]) && $this->form_data[email] != '') {
			// Pregmatch to email.
		}
		
		// URL validation
		if (isset($this->form_data[url]) && $this->form_data[url] != '') {
			if (!preg_match("/^http:\/\//", $this->form_data[url])) {
				$this->form_data[url] = 'http://' . $this->form_data[url];
			} else {
				// Check for CURL
				// If we have CURL, try a URL lookup
			}
		}
		
		$error = nl2br($error);
		$this->interface_data[error] .= $error;
		$this->theme->assign('idata', $this->interface_data);
		if ($error == '' ) { return 1; } else { return 0; }
	}
}

/*-----------------------------------------------------------------------------
** Interface Initialization
**    The interface class itself is now initialized, the operation variable 
**    used here is always included from the user interface GET or POST request.
**    The Interface constructor creates the global interface variable which 
**    makes it available to the other classes which are initialized by the
**    Initialize method. We cannot initialize in the constructor, as the object
**    is not actually instantiated yet.
*/
isset ($searchby) ? $operation .= ".$searchby" : '';
isset ($form__operation) ? $operation .= ".$form__operation" : '';
$gui = new LocalInterface();
/*    DarkPortal institutes a common set of CGI parameters which should always
**    be used when performing search or lookup operations for user data. The
**    basic method to create a search pattern is handled by three basic lookup
**    variables:
**    - searchby ==> the search lookup table for the query.
**    - searchid ==> the id to use when performing the search.
**    - searchname ==> If the search needs to lookup on an alternate column, 
**      indicate a name for that column.
**    In future releases, this API will be expanded to allow for ranged 
**    searching, and joins on multiple tables. Currently, the search function
**    is implemented directly in the Interface class. This will be changed to
**    be moved to the mainfile.
*/
$searchid = $userinfo[username];
$searchby = "users";
$searchname = "username";
$interface_cols = array('id', 'username', 'name', 'alias', 'email', 'fakeemail', 'url', 'locale_city', 'locale_country', 
		    'icq', 'aim', 'yim', 'msnm', 'occupation', 'interests', 'sig', 'bio', 'birth_date');
$gui->Initialize();

/*-----------------------------------------------------------------------------
** Operation Logic
**    Below is a sample of operational logic, which determines what the operation
**    is to be performed, and attempts to perform methods within the GUI to 
**    enact requests by the user. Note that the interface itself will automatically
**    retrieve the appropriate user interface layout blocks from the database 
**    during initialization. User interface methods should never generate output,
**    they should only extract or place data in the database based upon the 
**    user's request.
*/
if (preg_match("/^search/i", $operation)) {
  $gui->search();
} 
elseif (preg_match("/^submit/i", $operation)) {
  // It is typical to set the form data into the interface class if the operation
  // is a submission of information. Form data fields typically take on the form
  // of form_<dataitem>. This transposition may be modified in future releases.
  isset($form_id) ? $gui->form_data[id] = $form_id : '';
  $gui->form_data[name] = $form_name;
  // If an operation does not require input validation, it is placed first.
  if ($form__operation == 'delete') { 
    $gui->doDelete();
  } elseif ($gui->validateForm()){ 
    // Once the form data is placed, it can be validated. 
    // The validation is interface dependent.
    if ($form__operation == 'edit') {
      $gui->doEdit();
    } elseif ($form__operation == 'add') {
      $gui->doAdd();
    } elseif ($form__operation == 'copy') {
      $gui->doCopy();
    }
  } else {
    if ($form__operation == 'edit') {
      $gui->operation = 'edit';
      $gui->Initialize();
      $gui->edit();
    } elseif ($form__operation == 'add') {
      $gui->operation = 'add';
      $gui->add();
    } else {
      $gui->unknownError();
    }
  }
} elseif (eregi("^add", $operation)) {
  $gui->add();
} elseif (eregi("^edit", $operation)) {
  $gui->edit();
} elseif (eregi("^delete", $operation)) {
  $gui->delete();
} elseif (eregi("^copy", $operation)) {
  $gui->copy();
}

/*-----------------------------------------------------------------------------
** Interface Display
**    Once the operations (if any) are complete, the display method is called.
**    this enacts the object's theme to display its appropriate blocks.
*/
print $gui->display();
